; §§§   §§  §  § §    § §  § §§§§  §  §§§   §§§   §§  §§§§§ §  §§  §  §  §§§
; §  § §  § §  § §    § §§ § §  §  § §      §  § §  §   §   § §  § §§ § §
; §§§  §§§§ §  § §    § § §§ §§§§     §§    §§§  §  §   §   § §  § § §§  §§
; §    §  § §  § §    § §  § §  §       §   §    §  §   §   § §  § §  §    §
; §    §  §  §§  §§§§ § §  § §  §    §§§    §     §§    §   §  §§  §  § §§§

;A game by Leosoft for CPCRetrodev 2023

;Z80 grammar for compilation in WinApe

; Sorry. I didn't have the time to clean up this mess. If you really want to
; read this, do it at your own risk.


org #40
run #40
nolist

;##############
;#            #
;# CONSTANTS  #
;#            #
;##############

;FOR SELF-MODIFYING CODE
SMC_PLACEHOLDER equ 0

;Z80 OPCODES:
;------------
INCB equ #04
INCC equ #0C
DECB equ #05
DECC equ #0D
OP_SCF equ #37
OP_CPE equ #BB

;GAME SCEEN MEASUREMENTS:
;------------------------
SCREEN_TOTAL_WIDTH equ 86
WID_TILES equ 32
HGT_TILES equ 20
TOTAL_TILES equ HGT_TILES*WID_TILES
WID_BYTES equ WID_TILES*2
HGT_BYTES equ HGT_TILES*8
WID_UNITS equ WID_TILES*4
HGT_UNITS equ HGT_TILES*4
LINEBREAK_OFFSET equ SCREEN_TOTAL_WIDTH-WID_BYTES

;SPRITE MEASUREMENTS:
;--------------------
PAU_WIDTH equ 8
PAU_X_MAX equ WID_UNITS-PAU_WIDTH
PAU_HEIGHT equ 12
PAU_Y_MAX equ HGT_UNITS-PAU_HEIGHT+4

;TILES:
;-----
UPDATED_TILES_PER_FRAME equ 4
AMT_FRAMES_PER_TILE_CYCLE equ 8
TOTAL_ANIMATED_TILES equ UPDATED_TILES_PER_FRAME*AMT_FRAMES_PER_TILE_CYCLE

TILE_FG_BLACK equ #00
TILE_BG_BLACK equ #40
TILE_MAX_FG equ #1F
TILE_MIN_LADDER equ #20
TILE_MAX_LADDER equ #23
TILE_MIN_BG equ #24
TILE_MIN_ANIMATED equ #50
TILE_ITEM_MIN equ #58
TILE_ITEM_MAX equ #67
TILE_COMB_MIN equ #F0
TILE_COMB_MAX equ #FF
MAX_ITEMS_PER_STAGE equ 16


TILESET_OFFSET_40 equ #00
TILESET_OFFSET_C0 equ #80

; SPRITES:
SP_FRONT equ _pau_sprs/#100
SP_CLIMB equ SP_FRONT+1
SP_SIDE equ SP_FRONT+2
SP_JUMP equ SP_FRONT+4

;SP_LEVEL_40 equ #00
SP_LEVEL_C0 equ #50
SP_LEVEL_MASK equ #A0

JUMP_STEPS_BIGJUMP equ 8
JUMP_STEPS_DEFAULT equ 4

MAP_HI equ _stagemap/#100

;enemies
MAX_ENEMIES_PER_STAGE equ 4
LGTH_ENEMY_DATA equ 10

;offsets for the current enemies data table
ENE_SPEED equ 0			;0=enemy dead, -1=enemy falls, >-1 enemy on ground
ENE_COUNTER equ 1		;counter for update ()
ENE_OLD_MOV equ 2		;old position on moving axis
ENE_CURR_MOV equ 3		;current pos on moving axis
ENE_FIX_COORD equ 4		;fixed coord (x if vert, y if hori)
ENE_SPRITE equ 5		;sprite number, bit 7=axis (vert/hori)
ENE_DIRECTION equ 6		;offset for the moving coord
ENE_MIN_MOV equ 7		;minimum of the moving coord
ENE_MAX_MOV equ 8		;maximum of the moving coord

POTION_DURATION equ #2C	;no idea yet

MODE_DRAW_SCREEN equ 0	;0 = don't update sprites, leave gfx source addresses when switching screen
MODE_MENU equ 1			;(freeze everything in the game screen)
MODE_TIMESTOP equ 2		;only update main character sprite
MODE_JUMPNRUN equ 3		;update all sprites and animated tiles

PAU_DYING_DURATION equ 42

INVENTORY_SIZE equ 12
AMT_STAGES equ 32


;###########
;#         #
;# MACROS  #
;#         #
;###########


macro Add_A_to_HL:add l:ld l,a:jr nc,$+3:inc h:mend
macro Add_A_to_DE:add e:ld e,a:jr nc,$+3:inc d:mend
macro Add_A_to_BC:add c:ld c,a:jr nc,$+3:inc b:mend
macro Add_to_reg val,reg:ld a,val:add reg:ld reg,a:mend
macro Adc_treg val,reg:ld a,val:adc reg:ld reg,a:mend

macro load ;copy (hl) to (de) using a. like ldi, but w/o increment
	ld a,(hl):ld (de),a:mend


macro Draw_transp_byte reg
	;draw the bitmask in reg 0-transparent to scraddr in de
	;h has to contain the hibyte of the andmask table already
	ld l,{reg}:ld a,(de):and (hl):or l:ld (de),a:mend

macro Draw_spr_byte reg
	;hl=source (gets incremented), de=screen dest (stays same)
	;reg contains the transparency mask
	;gfx value is only read if needed
	inc reg:jr z,@next		;-1 -> don't draw anything
	dec reg:ld a,(de):and reg:or (hl):ld (de),a
	@next:inc l:mend

macro Draw_tile_line_l2r
	;draw 1 line (2 bytes) of a tile from left to right
	pop bc:ld (hl),c:inc l:ld (hl),b:mend

macro Draw_tile_line_r2l
	;draw 1 line (2 bytes) of a tile from right to left
	pop bc:ld (hl),b:dec l:ld (hl),c:mend

macro Draw_m_tile_line_l2r
	;one line of a mirrored tile left 2 right
	pop bc:ld l,b:load:inc e:ld l,c:load:mend

macro Draw_m_tile_line_r2l
	pop bc:ld l,c:load:dec e:ld l,b:load:mend

macro Limit_reg val,reg
	;checks if register is higher than value and resets to val if so
	ld a,val:cp reg:jr nc,$+3:ld reg,a:mend

;########################
;#                      #
;# PROVISORIC MAIN PROG #
;#                      #
;########################

	call initialize

game_start_page:
	call start_page
S_selected_game_input equ $+1
	ld hl,_keymap_arrows
	ld (S_active_input_device),hl

game_start_new_game:
	call reset_game_variables

gameloop_start_from_home:
	call draw_menu
	call erase_gamescreen

	ld a,-1
	ld (S_pau_coords_in_new_stage),a
	ld a,STAGE_HOME
	ld (S_curr_stage),a
	ld hl,STAGE_HOME_POS+_stages_layout
	ld (S_curr_stage_position),hl

gameloop_stage_update:
	ld hl,message_byte
	res 2,(hl)
	call set_mode_zero
	call stage_unpack
	call stage_add_gems
	call stage_init_items
	call stage_make_animated_list
	call stage_init_enemies
	call draw_stagemap
	call set_mode_jumpnrun
	call pau_set_new_coords
	ld a,(S_curr_stage):cp 29
	jp z,game_end

gameloop:
	ld a,(display_mode)
	cp MODE_MENU
		jr z,menuloop_nokey
	ld hl,message_byte
	bit 0,(hl)
		call nz,items_pick_up_item
	ld hl,message_byte
	bit 2,(hl)
		jr nz,gameloop_stage_update
	bit 3,(hl)
		call nz,erase_potion_icon_countdown_and_res_message_bit
	bit 7,(hl)
		call nz,pau_check_lives
	call enemies_drop_if_necess
	call mirror_pau_if_necess
	call mirror_ene_if_necess
	call check_collision
	call potion_check_countdown
	call draw_gems_display

	jr gameloop


pau_looses_life:
	ld a,SFX_DIE
	call snd_playsfx

	ld a,PAU_DYING_DURATION
	ld (S_pau_dies_countdown),a
	xor a
	ld (S_potion_countdown),a
	ret

pau_check_lives:
	res 7,(hl)
	call draw_lives_display
	ld a,(S_pau_lives):or a
ret nz
;game over
	pop hl
	call menu_game_over_alert
	call erase_gamescreen
	jp game_start_page


menuloop_nokey:
	call input_wait_no_key
menuloop:
	ld a,(display_mode)
	cp MODE_MENU
		jr nz,gameloop
	menu_stay:
	call display_cursor
	call input_wait_key_after_no_key
	push af
	call erase_cursor
	pop af						;input
	call menu_input_process
	jr menuloop

menu_input_process:
	ld hl,cursor_row
	ld c,(hl)
	ld b,(iy+24)				;x value, was set before
	rra:jr nc,$+3
		dec b
	rra:jr nc,$+3
		inc b
	rra:jr nc,$+3
		dec c
	rra:jr nc,$+3
		inc c
	rra:jr nc,menu_check_fire2
		dec c
			jp m,brew_potion
		;not potion book:
		dec c:dec c
			jr z,menu_top_row
			;drop
			call input_wait_no_key
			ld a,b
			jp items_drop_from_inv
	menu_check_fire2:
	rra:jr nc,menu_check_coords
		call input_wait_no_key
		jp set_mode_jumpnrun

	menu_check_coords:
		;make y go round (4 -> 0, -1 -> 3)
		ld a,c:and 3:ld c,a
		;check if x>max
		ld a,(iy+CURSOR_MAX_COL)			;max value
		cp b
		jr z,menu_dec_b_and_write_coords
		;check if x<0
		inc b
		jr z,menu_write_coords
		menu_dec_b_and_write_coords:
		dec b
		menu_write_coords:

		ld (iy+CURSOR_COLS),b
		ld hl,cursor_row
		ld (hl),c
		dec c:jp m,draw_potion_recipe
	ret

menu_top_row:
	dec b:jr z,menu_exit
	dec b:jp z,menu_help

menu_sacrifice_life:
	ld hl,_alert_sacrifice
	call menu_confirm_alert
	call z,pau_looses_life
	jp set_mode_jumpnrun

menu_exit:
	ld hl,_alert_exit_game
	call menu_confirm_alert
	jp nz,set_mode_jumpnrun
	call erase_gamescreen
	jp game_start_page

menu_help:
	call set_mode_zero
	call draw_help_screen
	call input_wait_key_after_no_key
	call draw_stagemap
	jp set_mode_jumpnrun

menu_confirm_alert:
	ld a,SFX_QUESTION:call snd_playsfx
	;hl=info text to confirm
	;returns z if confirmed
	ld de,_alert_confirm
menu_draw_alert_and_return:
	call menu_draw_alert_box
	push af
	call draw_stagemap
	pop af
	cp #40	;esc
ret

menu_game_over_alert:
	call set_mode_zero
	;call erase_gamescreen
	ld hl,_alert_gameover
	ld de,_alert_presskey
	jp menu_draw_alert_box

menu_draw_alert_box:
	;hl=message line 1, de=message line 2
	;returns a=input
	call set_mode_zero
	push de:push hl
	ld hl,ALERT_BOX_ORG
	ld bc,ALERT_BOX_SIZE
	call erase_area
	ld b,30
	ld hl,ALERT_BOX_ORG+1:call draw_hori_line
	ld hl,#6317:call draw_hori_line
	ld b,28
	ld hl,ALERT_BOX_ORG:call draw_vert_line
	ld hl,ALERT_BOX_ORG+31:call draw_vert_line

	ld de,ALERT_TEXT_ORG
	;ld bc,#ffff
	ld a,15
	pop hl
	call draw_string
	ld de,ALERT_TEXT_ORG+SCREEN_TOTAL_WIDTH
	;ld bc,#0ff0
	ld a,6
	pop hl
	call draw_string
	jp input_wait_key_after_no_key


game_end:
	ld hl,write_slow
	ld (S_slow_writing),hl
	ld hl,_cpc_shop
	call draw_string_read_color_and_scraddr
	call input_wait_key_after_no_key
	ld hl,#40C0:ld bc,#1B50
	call erase_area
	ld a,(_gems_in_pocket)
	cp 3
	jr nc,game_won
	ld hl,_cpc_game_lost
	call draw_string_read_color_and_scraddr
	call input_wait_key_after_no_key
	ld hl,write_fast
	ld (S_slow_writing),hl
	jp game_start_page

	game_won:
	ld hl,_cpc_game_won
	call draw_string_read_color_and_scraddr
	call input_wait_key_after_no_key
	jp 0

pau_rebirth:
	xor a
	ld (S_collision_animated_tile),a		;reset collision marker
	ld (jump_counter),a						;reset jump
	ld (pau_drift),a
	ld hl,message_byte
	set 3,(hl)
	set 7,(hl)
	ld hl,S_pau_lives
	dec (hl)

pau_set_new_coords:
S_pau_coords_in_new_stage equ $+1
	ld hl,#FFFF
	ld (pau_coords),hl
	ld (pau_coords_old),hl
ret

process_pau_mvt:
	ld hl,(pau_coords)
	ld (pau_coords_old),hl
	ld hl,S_pau_dies_countdown
	dec (hl)
	jr z,pau_rebirth
ret p

	inc (hl)				;restore to 0

	ld hl,wait_frame_after_picking
	dec (hl)
ret z
	inc (hl)

	ld ixl,SP_FRONT

	;check if fire 1 or 2 was pressed
	ld a,(player_input)
	check_fire1:
	bit 5,a
	jr z,check_fire2
	;chek if potion active, only go to menu if not
		ld a,(S_potion_countdown)
		or a
		jp m,set_mode_menu
	check_fire2:
	bit 4,a
	jr z,ppm_check_jump
		ld hl,message_byte
		set 0,(hl)
		ret

ppm_check_jump: ;check if pau is currently jumping
	ld hl,jump_counter
	xor a:cp (hl)
	jr z,ppm_check_ladder_and_fall
	    ;jump is active:
		dec (hl)
		ld hl,pau_drift
		;b=offset x, c=offset y
		ld b,(hl)					;drift
		ld c,-2
		ld ixl,SP_JUMP
		jr ppm_check_borders

	ppm_check_ladder_and_fall:
	ld bc,(pau_coords)
	inc c
	call check_tiles_behind_paulina_bottom_row

	;e=blocking tiles, d=ladders
	xor a:cp d					;is there a ladder?
	jr nz,ppm_reset_drift		;if yes, dont check fall

	ppm_check_fall:
	cp e						;are there blocking tiles?
	ld hl,S_pau_falling
	jr nz,ppm_reset_drift		;not falling
	;falling:
		ld (hl),OP_SCF			;set falling flag
		ld ixl,SP_JUMP			;set sprite to jump (=fall)
		ld c,2					;fall 2px
		ld hl,pau_drift
		ld b,(hl)
		ld hl,pau_y_coord
		res 0,(hl)				;make y even, or we might get stuck in ground
		jr ppm_check_borders

	ppm_reset_drift:
	ld (S_pau_falling),a
	ld (pau_drift),a


	ppm_process_input:
	ld ixh,d						;backup ladder count
	ld bc,(pau_coords)
	;now check if ladder behind feet
	call check_tiles_behind_paulina_bottom_row

	ld a,(player_input)
	or a
		jp z,ppm_calc_sprite

	ld ixl,SP_SIDE
	ld bc,0							;reset offset
	ld hl,pau_spr_orientation
	ld (hl),b						;reset orientation
	rra:jr nc,$+4
		;left:
		dec b:dec (hl)				;look left
	rra:jr nc,$+3
		;right:
		inc b
	rra:jr nc,ppm_inp_down
		;up
		inc d:dec d
		jr z,ppm_start_jump			;if no ladders behind feet, jump
			;climbing:
			ld ixl,SP_CLIMB:dec c
			jr ppm_check_borders
		ppm_start_jump:
			ld a,SFX_JUMP
			call snd_playsfx
			ld ixl,SP_JUMP:dec c
		S_jump_height equ $+1
			ld a,SMC_PLACEHOLDER
			ld (jump_counter),a
			ld a,b:ld (pau_drift),a
			xor a					;shortcut for jump to ppm_check_borders
	ppm_inp_down:
	rra:jr nc,ppm_check_borders
		dec ixh
		jp m,ppm_check_borders		;if there are no ladders, ignore
			ld ixl,SP_CLIMB:inc c


	ppm_check_borders:
	ld hl,(pau_coords)
	ld de,0     					;reset offset for level change

	ld a,l:add c					;apply movement to y
	ld c,a							;save in c for later
	jp p,ppm_check_stage_down
		;change to stage up
		ld de,-STAGES_LAYOUT_WID
		ld l,PAU_Y_MAX-3
		jr stage_try_change
	ppm_check_stage_down:
	cp PAU_Y_MAX
	jr c,ppm_check_stage_left
		ld l,e
		ld de,STAGES_LAYOUT_WID
		jr stage_try_change
	ppm_check_stage_left:
	ld a,h:add b					;apply movement to x
	ld b,a							;save in b for later
	jp p,ppm_check_stage_right		;if x<0
		dec de
		ld h,PAU_X_MAX
		jr stage_try_change
	ppm_check_stage_right:
	cp PAU_X_MAX+1
	jr nz,ppm_check_if_movement_possible		;if x>max
		ld h,e
		inc de
		jr stage_try_change

	ppm_check_if_movement_possible:
	push bc ;target coords
	call check_tiles_behind_paulina
	pop hl
	xor a
S_potion_jump_walking_walls:
	cp e ;nop if wall potion active, so z stays set
	jr z,ppm_write_new_coords		;no blocking tiles

		;movement blocked:
		ld (jump_counter),a 		;stop jump
		ld (pau_drift),a 			;stop sideways drift
		ret

	ppm_write_new_coords:
	ld (pau_coords),hl

	ppm_calc_sprite:
	;calculate which sprite to display
	ld a,ixl
	ld e,a

	cp SP_SIDE
		jr nz,ppm_calc_sprite_branch
		ld a,7:and h
		jr nz,ppm_calc_spr_sideways
		jr ppm_calc_spr_sideways

	ppm_calc_sprite_branch:
	sub SP_CLIMB
	jr c,ppm_write_spr			;-> front sprite
	jr nz,ppm_calc_spr_sideways	;-> side or jump
		;climbing
		xor a
		bit 2,l					;flip "orientation" every 4 y's
		jr z,ppm_climb_after_flip
			ld a,3:and l
			jr nz,ppm_climb_after_flip-2
			ld a,-1
		ppm_climb_after_flip:
		ld (pau_spr_orientation),a
		jr ppm_write_spr

	ppm_calc_spr_sideways:
	ld a,(pau_spr_orientation)
	xor h					;combine with x odd/even
	rra:jr c,$+3			;if result even, show alternative sprite
		inc e

	ppm_write_spr:
	ld a,e
	ld (S_pau_curr_spr),a
ret

stage_try_change:
	ld (S_pau_coords_in_new_stage),hl
S_curr_stage_position equ $+1
	ld hl,_stages_layout+STAGE_HOME_POS
	add hl,de
	ld a,(hl):or a
ret m
	ld (S_curr_stage),a
	ld (S_curr_stage_position),hl
	ld hl,message_byte
	set 2,(hl)
	ret



;#################
;# MENU GRAPHICS #
;#################

draw_menu:
	;reset cursor settings
	ld hl,curr_potion_recipe
	ld b,4:ld (hl),0:inc hl:inc hl:djnz $-4

	call erase_menu_area
	call draw_book
	ld de,MENU_UPPER_ROW+1:ld hl,_menu_suicide:call draw_symbol
	ld de,MENU_UPPER_ROW+7:ld hl,_menu_exit:call draw_symbol
	ld de,MENU_UPPER_ROW+13:ld hl,_menu_help:call draw_symbol
	ld de,GEMS_ICON_ORG:ld hl,_tiles+#ac0:call draw_tile_slow_direct_gfxaddr
	ld de,LIVES_ICON_ORG:ld hl,_tiles+#d00:call draw_tile_slow_direct_gfxaddr
	call draw_book_first_page
	call draw_inventory

	call draw_lives_display
	call draw_gems_display
	jp draw_menu_borders
ret

draw_inventory:
	ld hl,_inventory
	ld b,INVENTORY_SIZE
	ld de,INVENTORY_SCRADR_1
	draw_inventory_loop:
		ld a,6:cp b
		jr nz,$+5
			ld de,INVENTORY_SCRADR_2
		ld a,(hl)
		push bc:push hl
		call draw_item
		res 7,d
		pop hl:pop bc
		inc de:inc de:inc de
		inc hl
		djnz draw_inventory_loop
		ret


draw_potion_recipe:
	call erase_book_pages

	;a=potion number (0=title, max 16 recipes)
	ld a,(curr_potion_recipe)
	or a
	jr z,draw_book_first_page



	;potion symbol
	dec a
	ld de,BOOK_POTION_SYM_ORG
	call draw_potion_symbol

	ld d,0:ld e,iyl
	ld hl,_all_potions_recipes
	add hl,de
	ld iy,_book_item_syms
	ld ixl,6
	dpr_ingred_loop:
		ld e,(iy):ld d,(iy+1)
		inc iy:inc iy
		ld a,(hl):inc hl
		push hl
		call draw_item
		pop hl
		dec ixl
		jr nz,dpr_ingred_loop
	ret

draw_book_first_page:
	call erase_book_pages
	ld de,BOOK_ITEMS_ORG+86
	ld hl,_book_flask
	jp draw_symbol



erase_book_pages:
	ld hl,BOOK_POTION_SYM_ORG
	ld bc,#040F
	call erase_area
	ld hl,BOOK_ITEMS_ORG
	ld bc,#0620
	jp erase_area


draw_book:
	ld de,BOOK_SCRADDR+#8000
	ld hl,_book_open
	call draw_book_edges
	;draw book pages
	ld c,34
	draw_book_pages_lp:
		push hl
		call draw_book_line
		pop hl
		dec c
		jr nz,draw_book_pages_lp

draw_book_edges:
ld c,6
	draw_book_edges_lp:
		call draw_book_line
		dec c
		jr nz,draw_book_edges_lp

draw_book_line:
	push de
	call draw_book_line_halve
	call draw_book_byte
	call draw_book_line_halve
	pop de
	call nextline_de
	ret

draw_book_line_halve:
	call draw_book_2_bytes
	ld b,5
	draw_book_line_lp:
		call draw_book_byte
		dec hl
	djnz draw_book_line_lp
	inc hl
draw_book_2_bytes:
	call draw_book_byte

draw_book_byte:
  ;hl=source, de=dest (40)
  ;both return incremented
	push bc
	res 7,d
	ld a,(hl)
	ld b,a
	rrd
	and #F0
	xor (hl)
	ld (hl),b
	ld (de),a
 	set 7,d
    	ldi
	pop bc
ret

redraw_tile:
	;redraws the tile at address de in the stagemap
	;only de conserved
	push de
	ld a,(de)
	push af
	call conv_mapaddr2scraddr
	res 7,d
	pop af
	call draw_tile_slow
	pop de
	ret

draw_item:
	add TILE_ITEM_MIN
	cp TILE_ITEM_MIN-1
	jr nz,$+3
		xor a

draw_tile_slow:
	;opaque. Adress can be everywhere.
	;draws at both screens @ once
	;input a=item number de=screen address @40
	or a
		jr nz,$+4:ld a,#40
	call conv_tileidx2gfxaddr_base
draw_tile_slow_direct_gfxaddr:
	call draw_tile_slow_do
	set 7,d

	draw_tile_slow_do:
		push de
		ld bc,_tile_lines_offsets
	draw_tile_slow_lines:
		load
		inc de:inc l
		load
		dec de
		;next line in source
		ld a,(bc)
		add l:ld l,a
		;next line on screen
		call nextline_de
		inc bc
		ld a,_tile_lines_offsets+8:cp c
		jr nz,draw_tile_slow_lines
		pop de
ret




draw_potion_symbol:
	;a=potion number, de=scraddr (40-based)
	add a:add a:add a		;*8
	ld iyl,a				;backup for ingredient list
							;(if called from recipe)
	ld l,a:ld h,0
	add hl,hl:add hl,hl
	ld bc,_potion_symbols
	add hl,bc

draw_symbol:
	ld a,(hl)
	ld (S_potion_color_mask1),a
	inc hl
	push de:push hl
	inc hl
	call draw_symbol_do
	pop hl:pop de:set 7,d
	ld a,(hl)
	ld (S_potion_color_mask2),a
	inc hl

draw_symbol_do:
	ld b,a		;b=mask
	ld ixh,15
	dps_line:
		push de
		ld ixl,2
		dps_8px:
			call draw_eight_pixels
			inc de
			dec ixl:jr nz,dps_8px
		pop de
	dec ixh:ret z
		call nextline_de
		jr dps_line

draw_string_read_color_and_scraddr:
	ld a,(hl):inc hl

draw_string_read_scraddr:
	ld e,(hl):inc hl
	ld d,(hl):inc hl

draw_string:
	;draws a #ff-terminated string (caps only)
	;hl=source, a=color (bc=color masks,)
	; de=scraddr
	;hl returns at address AFTER 0

	;convert color in a to color mask in bc
	push hl
	or a:sbc hl,hl
		ld b,4
		draw_string_convert_color:
		rrca:adc hl,hl:rlca:adc hl,hl
		rrca:adc hl,hl:rlca:adc hl,hl
		rra
			djnz draw_string_convert_color
	ld b,h:ld c,l
	pop hl
draw_string_input_bc:
	push de				;save for cr
draw_string_loop:
	ld a,(hl):inc hl
	cp "\"
	jr nz,draw_string_not_cr
		pop de
		ld a,SCREEN_TOTAL_WIDTH
		Add_A_to_DE
		push de
		jr draw_string_loop
	draw_string_not_cr:
	push af:push hl:push de:push bc
	call draw_letter
	write_fast equ #0001
	write_slow equ #7607
S_slow_writing equ $+1
	ld b,1:nop:djnz $-1

	pop bc
	pop de:inc de
	pop hl
	pop af
	add a		;end if bit 7
jr c,draw_string_finish
	cp 12		;(char 1,2,3,4,5)
	jr c,$+3
		inc de
	jr draw_string_loop
draw_string_finish:
	pop de
	ret


draw_digit:
	add "0"
draw_letter:
	;a=ascii, de=scraddr, bc=masks
	;draws to both screens
	push bc
	ld l,a:add a:add l:add a			;*6
	add _font:ld l,a
	ld a,_font/#100
	adc 0:ld h,a						;hl=source
	push hl:push de						;backup
	call draw_letter_lines
	pop de:pop hl
	pop bc:ld b,c
	set 7,d

	draw_letter_lines:
	ld ixl,6
		draw_letter_lines_lp:
		call draw_eight_pixels
		dec de
		call nextline_de
		dec ixl
		jr nz,draw_letter_lines_lp
ret


draw_eight_pixels:
	;input hl=source (1 byte), de (scraddr, 2 bytes), b=mask
	;output hl+=1, de+=1
	;read the byte from (hl), convert it to 2 mode 1 bytes & write to de
	ld a,(hl)			;get the monochrome byte
	ld c,a				;& back it up
	rld					;double left nibble
	and b				;convert to 1st 4-color-byte
	ld (de),a:inc de	;draw it & advance scraddr
	ld a,(hl)			;get the other doubled nibble
	and b				;convert to 2nd 4-color-byte
	ld (de),a			;draw it
	ld (hl),c			;restore original byte
	inc hl
ret


start_menu_toggle_snd:
	ld de,"F"*#100+"F"+#80 			;"(O)FF"
	ld ixh,COLOR_BROWN
	ld a,d:cp (hl)
	ld a,#c9
	jr nz,$+9
		xor a
		ld de,"N"*#100+" "+#80		;"(O)N "
		ld ixh,COLOR_GREEN
	ld (bc),a						;patch code
	ld (hl),d:inc hl:ld (hl),e
	ld de,-5:add hl,de
	ld a,ixh:ld (hl),a
	ld a,SONG_MENU
	call snd_play_song
	jr start_page_redraw_menu

start_page:
	ld hl,_keymap_func_keys
	ld (S_active_input_device),hl
	call erase_menu_area
	call set_mode_zero
	ld a,32
	ld (S_curr_stage),a
	call stage_unpack
	call draw_stagemap
	ld hl,_text_startscreen:ld a,#15
	call draw_string_read_color_and_scraddr
start_page_redraw_menu:
	ld hl,_text_start_menu_1:ld a,#9
	ld ixh,9
		draw_start_menu_loop:
		ld a,6
		call draw_string_read_color_and_scraddr
		dec ixh
		jr nz,draw_start_menu_loop
start_page_getkey:
	call input_wait_key_after_no_key
	rra									;6
		jp c,0
	rra									;5
		;sfx on/off
		ld bc,S_sfx_toggle_code			;address to change code
		ld hl,S_sfx_toggle_text			;address to change text
		jr c,start_menu_toggle_snd

	rra									;4
		;music on/off
		ld bc,S_music_toggle_code
		ld hl,S_music_toggle_text
		jr c,start_menu_toggle_snd

	rra									;3
		;emulator arrows
		ld hl,_keymap_arrows
		ld bc,color_arrow
		jr c,start_menu_save_input
	rra									;2
		;CPC keys
		ld hl,_keymap_CPC
		ld bc,color_cpc
		jr c,start_menu_save_input
	rra									;1
		;joystick
		ld hl,_keymap_joystick
		ld bc,color_joy
		jr c,start_menu_save_input
	rra 								;0
		ret c
	jr start_page_getkey

	start_menu_save_input:
	ld a,COLOR_BROWN
	ld (color_cpc),a:ld (color_joy),a:ld (color_arrow),a
	ld a,COLOR_GREEN
	ld (bc),a
	ld (S_selected_game_input),hl
	jr start_page_redraw_menu


draw_help_screen:
	call erase_gamescreen
	ld de,#6056
	ld a,0
	ld hl,_potion_labels
		draw_help_potion_symbols:
		push af
		push hl
		push de
		call draw_potion_symbol
		pop de
		pop hl
		push de
		ld a,5:add e:ld e,a
	S_potion_color_mask2 equ $+1
	S_potion_color_mask1 equ $+2
		ld bc,COLOR_BLUE
		call draw_string_input_bc
		pop de
		call nextline_de
		call nextline_de
		ld a,86*2:Add_A_to_DE
		pop af
		inc a
		cp 8
		jr nz,$+5
		ld de,#606D
		cp 15
		jr nz,draw_help_potion_symbols

	ld a,10:ld hl,_text_help
		draw_help_screen_loop:
		push af
		call draw_string_read_color_and_scraddr
		pop af
		dec a:jr nz,draw_help_screen_loop
	ret

;####################
;#                  #
;# DRAWING ROUTINES #
;#                  #
;####################






; TILES & stagemap
;-----------------


draw_tile_opaque_curr_screen:
S_tileset_screen_offset equ $+1
	ld l,SMC_PLACEHOLDER					;offset for c0/40 gfx
draw_tile_opaque_manual_screen:
	;l must already contain the tileset offset

;STACK ABUSE -> MUST BE CALLED INSIDE INTERRUPT OR BETWEEN DI&EI
	;opaque tile of 2x8 inside 1 charline
	;duration incl. call:
	;126 standard, 176 mirrored, 74 empty
	;a=tile index  de=scradd
	;de returns @ origin of the right neighbor tile
	;calculate source address of tile (max index=127, bit 7 set=mirror!)

	;check if black tile (#00 and #40)
	;ld b,a:;res 6,b
	cp #40:jr z,draw_tile_empty
	or a:jr z,draw_tile_empty

	call conv_tileidx2gfxaddr
	;hl=tile source, de=scraddr, b=index
draw_tile_opaque_manual_source:
	di
	ld (S_draw_tile_sp),sp
	ld sp,hl
	jr nz,draw_tile_mirrored		;mirrored
	ld h,d:ld l,e
	Draw_tile_line_l2r:set 3,h:Draw_tile_line_r2l:set 4,h
	Draw_tile_line_l2r:set 5,h:Draw_tile_line_r2l:res 4,h
	Draw_tile_line_l2r:res 3,h:Draw_tile_line_r2l:set 4,h
	Draw_tile_line_l2r:res 5,h:Draw_tile_line_r2l
	jr finish_tile_opaq_get_sp

draw_tile_empty:
	xor a
	ex de,hl
	ld de,#7FF
	ld b,8
		dte_loop:
		ld (hl),a:inc l
		ld (hl),a:add hl,de
		djnz dte_loop
	ld de,#c000
	add hl,de
	ex de,hl
	ret

draw_tile_mirrored: ;148
	;de=scradd, sp=source adr
	ld h,_lut_mirror/#100
	Draw_m_tile_line_l2r:set 3,d:Draw_m_tile_line_r2l:set 4,d
	Draw_m_tile_line_l2r:set 5,d:Draw_m_tile_line_r2l:res 4,d
	Draw_m_tile_line_l2r:res 3,d:Draw_m_tile_line_r2l:set 4,d
	Draw_m_tile_line_l2r:res 5,d:Draw_m_tile_line_r2l
	;fall through!

finish_tile_opaq_get_sp:
S_draw_tile_sp equ $+1
	ld sp,SMC_PLACEHOLDER
finish_tile_opaq:
	res 4,d				;go back to 1st scanline
	;inc e:;inc e			;and advance to next tile
ret



draw_stagemap:
;and create list of animated tiles

	ld de,#4000
	ld a,TILESET_OFFSET_40
	call draw_stagemap_do
	ld de,#c000
	ld a,TILESET_OFFSET_C0
draw_stagemap_do:
	ld (S_draw_stagemap_screen_offset),a
	ld hl,_stagemap
	ld c,HGT_TILES
	dtm_lpo:
		ld b,WID_TILES
		dtm_lpi:
			push bc
			ld a,(hl)
			inc hl
			push hl
			di
		S_draw_stagemap_screen_offset equ $+1
			ld l,SMC_PLACEHOLDER
			call draw_tile_opaque_manual_screen
			inc e:inc e
			ei
			pop hl:pop bc
			djnz dtm_lpi
		ld a,LINEBREAK_OFFSET
	Add_A_to_DE
	dec c:jr nz,dtm_lpo
ret


; SPRITES
;-----------------

draw_pau_spr:
S_pau_dies_countdown equ $+1
	ld a,SMC_PLACEHOLDER
	or a:jr z,draw_pau_sprite_do

	;flashing effect:
	bit 2,a
	ret z

draw_pau_sprite_do:
	ld ixl,9 ;10
	ld bc,(pau_coords)
	call conv_coords2scraddr
S_pau_spr_addr equ $+1
S_pau_curr_spr equ $+2
	ld hl,SP_SIDE*#100+SP_LEVEL_MASK
	ld a,SP_LEVEL_C0
	bit 7,d
	jr z,$+3		;check if screen @ c0
		xor a		;if not, change sprite source

	;hl=sprite mask table, a=offset (0 or #50), de=scraddr
	ld (S_spgarage_p),sp
	ld sp,hl
	ld l,a
	draw_pau_lp:
		pop bc:Draw_spr_byte c:inc e:Draw_spr_byte b:inc e
		pop bc:Draw_spr_byte c:inc e:Draw_spr_byte b
		set 3,d
		pop bc:Draw_spr_byte c:dec e:Draw_spr_byte b:dec e
		pop bc:Draw_spr_byte c:dec e:Draw_spr_byte b
		dec ixl
		jr z,draw_pau_end
		Add_to_reg 8,d
		add a						;check if last line of char
		jp m,draw_pau_lp
			Add_to_reg SCREEN_TOTAL_WIDTH,e
			Adc_treg #c0,d
		jr draw_pau_lp
	draw_pau_end:
S_spgarage_p equ $+1
	ld sp,SMC_PLACEHOLDER

	;Fall through
	;draw feet:
	ld a,h
	cp SP_JUMP
		ret nc			;jump sprites are shorter & feet included
	cp SP_CLIMB+1
		jr c,draw_p_setbc_for_feet	;if climb or base, always the same feet

	ld a,(pau_x_coord)
	bit 1,a

	ld a,l
	jr nz,draw_p_setbc_for_feet

	;48 -> E8
	;98 -> F0
	add #A0
	jr nc,$+4
	ld a,#F0
	ld l,a

	draw_p_setbc_for_feet:
	ld b,h:ld c,l
	ld ixl,2
	jr ds0t_start_with_nextline


draw_ene_sprs:
	;enemies:
	ld iy,_enemies_curr_stage
	ld ixh,MAX_ENEMIES_PER_STAGE-1
	draw_enemies_lp:
		call draw_ene_sprs_do
		ld a,LGTH_ENEMY_DATA
		add iyl:ld iyl,a
		dec ixh:jr nz,draw_enemies_lp


draw_ene_sprs_do: ;35
	ld a,(iy+ENE_SPEED)
	;0=deactivated -1=falling -2=on ground
	or a:ret z
	inc a:ret m
	dec a

	ld c,(iy+ENE_CURR_MOV)
	ld b,(iy+ENE_FIX_COORD)
	ld a,(iy+ENE_SPRITE)
	ld i,a				;backup
	rla 				;get axis bit in carry
	ld a,c				;save moving coord
	jr c,$+4			;horizontal movement -> exchange x & y
		ld c,b:ld b,a

	bit 7,(iy+ENE_DIRECTION)	;get right/left bit in z
	push af				;preserve orientation and moving coord
	call conv_coords2scraddr
	pop af
;calc lowbyte:
	;invert coord (?) if direction left to right
	;for correct shifted sprite
	jr nz,$+3
		cpl
	rlc d:rla:rrc d				;shift the current screen in
	add a:add a:add a:add a:add a	;5x left shift
	ld c,a
	ld a,i						;sprite hibyte
	res 7,a						;in case it's vertical
	add _ene_sprs/#100
	ld b,a
	ld ixl,4
	call draw_sprite_0_transparent
	ret




draw_sprite_0_transparent:
	;4 bytes (16px) wide, source has to be on one page and 4-aligned!
	;bc=source, de=dest, ixl=height (in double lines)
	ld h,_lut_transparency_masks/#100
		ds0t_l2r:
		ld a,(bc):Draw_transp_byte a
		inc e:inc c
		ld a,3:and c:jr nz,ds0t_l2r
	set 3,d
		ds0t_r2l:
		dec e
		ld a,(bc):Draw_transp_byte a
		inc c
		ld a,3:and c:jr nz,ds0t_r2l
ds0t_start_with_nextline:
	call nextline_de
	dec ixl
	jr nz,draw_sprite_0_transparent

ret

erase_pau_spr:
;THIS CAN CLEARLY BE OPTIMIZED

	;AND analyze background
	;bc=xy (tiles coords)
	xor a
	ld (S_item_behind_pau+1),a			;reset hibyte
	ld (S_collision_animated_tile),a	;reset anim collision

	ld bc,(pau_coords_old)
	;calculate how many cols and rows of tiles
	;have to be examined
	ld d,3					;"constant"
	ld a,b:and d
	ld a,d:jr nz,$+3
		dec a
	ld iyl,a
	ld a,c:and d
	cp d:jr c,$+3
		inc d				;3 lines or 4 if y mod 4 > 3
	ld iyh,d

	call conv_coords2tilepos
	call conv_tilepos2scraddr
	call conv_tilepos2mapaddr
	es_lines:
		ld a,iyl
		ld ixl,a
		es_tiles:
			push hl
			ld b,(hl)
			;check if animated tile
			ld a,%01111000
			and b:cp %1010000
			jr z,es_anim_tile_found
			;check if item (combination tile)
			ld a,b:cp TILE_COMB_MIN
			jr c,es_draw_tile
				ld (S_item_behind_pau),hl
			es_draw_tile:
			call draw_tile_opaque_curr_screen
			es_next_tile:
			inc e:inc e
			pop hl
			inc hl
			dec ixl
			jr nz,es_tiles
		ld a,SCREEN_TOTAL_WIDTH
		sub iyl:sub iyl
		Add_A_to_DE
		ld a,WID_TILES
		sub iyl
		Add_A_to_HL
		dec iyh
		jr nz,es_lines
ret

es_anim_tile_found:
;check if diamond
	ld a,b:and %01111110
	cp #56
	jr z,es_found_diamond
	push bc
	ld hl,S_collision_animated_tile
	;calc animation type
	ld a,7:and b:rra	;isolate tile index
	inc a				;+1 for djnz
	ld b,a
	ld a,#80
		rlca
		djnz $-1
	;1=water, 2=fire, 4=gas, 8=diamonds
	or (hl)				;OR it to the collision marker
	ld (hl),a			;(in case there are different types @ once)
	pop af
	jr es_draw_tile

es_found_diamond:
	ld (hl),#40				;empty place in map
	ld a,h:sub _stagemap/#100:ld h,a
	ld a,(S_curr_stage)
	add a:add a:add h
	ld h,a					;expected hibyte (l=expected lo)
S_curr_stages_gems equ $+1
	ld bc,_all_gems
		es_search_in_gem_list:
		ld a,(bc):cp l:inc bc
		jr nz,es_search_in_gem_list_next
		ld a,(bc):cp h
		jr z,es_gem_found_in_list
		;inc a:;jr z,$
		es_search_in_gem_list_next:
		inc bc
		jr es_search_in_gem_list
	es_gem_found_in_list:
	set 7,a:ld (bc),a		;deactivate by setting bit 7 of hibyte
	call add_one_gem
	ld a,SFX_GEM
	call snd_playsfx
	ld hl,wait_frame_after_picking
	inc (hl)
	ld a,#40					;star tile for short bling whe collecting
	jr es_draw_tile


erase_ene_spr:
	;erase enemy sprites
	ld iy,_enemies_curr_stage
ld ixh,MAX_ENEMIES_PER_STAGE-1
	era_ene_loop:
	call erase_enemy
	ld de,LGTH_ENEMY_DATA
	add iy,de
	dec ixh
	jr nz,era_ene_loop

erase_enemy:
	ld b,(iy+ENE_OLD_MOV)
	ld c,(iy+ENE_FIX_COORD)		;coords
	bit 7,(iy+ENE_SPRITE)		;hori/vert
	push af						;save z for later
	jr z,$+5					;swap x/y if vertical
		ld a,b:ld b,c:ld c,a
	call conv_coords2tilepos
	call conv_tilepos2scraddr
	call conv_tilepos2mapaddr
	pop af
	jr nz,ee_vertical
		;horizontal - 3 adjacent tiles
		ld ixl,3
		ee_tiles:
		call ee_draw_tile
		inc hl
		dec ixl:jr nz,ee_tiles
ret

	ee_vertical:
	;square of 4 tiles
		res 3,d:res 4,d
		ld ixl,2
		ee_vertical_lp:
		call ee_draw_tile
		inc hl
		call ee_draw_tile
		dec ixl
		ret z
		ld bc,WID_TILES-1:add hl,bc	;next line in tilemap
		ld a,SCREEN_TOTAL_WIDTH-4:Add_A_to_DE	;next line on screen
		jr ee_vertical_lp


	ee_draw_tile:
	push hl
	ld a,(hl)
	call draw_tile_opaque_curr_screen
	inc e:inc e
	pop hl
	ret

;**********
; MENU GFX
;**********


erase_cursor:
	ld bc,0
	jr draw_cursor
display_cursor:
	ld bc,#030c
draw_cursor:
	ld a,(cursor_row)
	add a:ld d,0:ld e,a
	ld iy,_cursor_values
	add iy,de
	ld l,(iy):ld h,(iy+1)
	ld e,(iy+CURSOR_WIDTH)
	or a:jr z,draw_cursor_do	;if 0'th row,
	;calc x pos of cursor by multiplying (width+1) with col
	ld d,(iy+CURSOR_COLS)
	xor a
		draw_cursor_mult:
		add e:inc a
		dec d
		jr nz,draw_cursor_mult
	Add_A_to_HL
	draw_cursor_do:
	push hl
	call do_draw_cursor
	pop hl
	set 7,h

	do_draw_cursor:
	call dcc_2_hori_edges
	sbc hl,de:call nextline_hl
	call dcc_2_side_edges
	push de
	ld e,(iy+CURSOR_HEIGHT):ld d,(iy+CURSOR_HEIGHT+1)
	add hl,de
	pop de
	call dcc_2_side_edges

	;& fall through
	dcc_2_hori_edges:
	call dcc_hori_edges
	res 3,h:add hl,de
	dcc_hori_edges:
	ld (hl),b:inc hl:ld (hl),c
	set 3,h
	ld (hl),c:dec hl:ld (hl),b
	ret

	dcc_2_side_edges:
	ld (hl),b:set 3,h:ld (hl),b
	add hl,de:inc hl
	ld (hl),c:res 3,h:ld (hl),c
	scf:sbc hl,de:set 3,h
	jp nextline_hl

draw_potion_timer:
	ld c,12:ld de,#0F0f
	ld hl,POTION_TIMER_SCRADDR
		draw_potion_timer_loop:
		push hl
		call draw_potion_timer_byte
		pop hl
		inc hl
		dec c
		jr nz,draw_potion_timer_loop
ret


update_potion_timer:
	;a=current countdown
	ld b,a
	srl a:srl a		;/4 -> bitmap position
	ld h,POTION_TIMER_SCRADDR/#100
	add POTION_TIMER_SCRADDR:ld l,a
	jr nc,$+3
		inc h
	ld a,b
	and 3		;how many pixels in the current bitmap
	ld d,_timer_bitmaps/#100
	add _timer_bitmaps:ld e,a
	ld a,(de)
	ld d,a:ld e,a

draw_potion_timer_byte:
	;actually, draw 4 bytes in a column
	ld b,4
		draw_potion_timer_byte_lp:
		call draw_mask_to_both_screens
		call nextline_hl
		djnz draw_potion_timer_byte_lp
	ret



draw_lives_display:
	ld de,LIVES_NUMBER_ORG
	ld bc,#0F0F
S_pau_lives equ $+1
	ld a,SMC_PLACEHOLDER
	jp draw_digit

draw_gems_display:
	ld hl,_gems_in_pocket
	ld de,GEMS_NUMBER_ORG
	ld b,3
		draw_gems_digit:
		push bc:push de:push hl
		ld a,(hl)
		ld bc,#0F0F
		call draw_digit
		pop hl:pop de:pop bc
		inc de:inc de
		inc hl
		djnz draw_gems_digit
	ret



draw_menu_borders:
	ld hl,_menu_hori_lines
	ld bc,#1306
		draw_hori_lines_loop:
		ld a,(hl):inc hl
		push hl
		ld h,(hl):ld l,a
		call draw_hori_line
		pop hl:inc hl
		dec c
		jr nz,draw_hori_lines_loop
	ld b,158
	ld hl,#4040:call draw_vert_line
	ld hl,#4054

draw_vert_line:
	ld de,#f00f
	call draw_mask_to_both_screens
	call nextline_hl
	push bc
	draw_vert_line_loop:
		ld de,%1001000100011001
		call draw_mask_to_both_screens
		call nextline_hl
		djnz draw_vert_line_loop
	pop bc
	ld de,#ffff


draw_mask_to_both_screens:
	;hl=dest @40, d=byte for #40, e=byte for #c0
	ld (hl),d:set 7,h
	ld (hl),e:res 7,h
	ret

draw_hori_line:
	push bc
	draw_hori_line_loop:
	push hl
	ld de,#f00f:call draw_mask_to_both_screens
	call nextline_hl:call nextline_hl:call nextline_hl
	ld de,#ffff:call draw_mask_to_both_screens
	pop hl
	inc hl
	djnz draw_hori_line_loop
	pop bc
	ret

;################
;#              #
;# CALCULATIONS #
;#              #
;################

nextline_de:
;calc the next scanline for address in de
;uses A
	Add_to_reg 8,d
	add a:ret m						;this way, it works for #40 and #C0 screen
	Add_to_reg SCREEN_TOTAL_WIDTH,e
	Adc_treg #c0,d
ret

nextline_hl:
;calc the next scanline for address in hl
;uses A
	Add_to_reg 8,h
	add a
ret m
	Add_to_reg SCREEN_TOTAL_WIDTH,l
	Adc_treg #c0,h
ret


add_one_gem:
	ld hl,_gems_in_pocket+2
		es_diamond_inc_loop:
		inc (hl)
		ld a,10:cp (hl)
		ret nz
		ld (hl),0
		dec hl
		jr es_diamond_inc_loop


; CONVERSIONS
;-------------


conv_coords2scraddr:
;input b=x , c=y (in 2 pixel units)
;output de=scraddr, bc stays intact
;af & hl ruined
	ld a,c:add a
	add _lut_scraddr:ld l,a
	ld h,_lut_scraddr/#100
	ld e,(hl):inc l				;get lowbyte of line
S_curr_screen_org equ $+1
	ld a,#40					;initial value!
	add (hl):ld d,a				;add screen org to hibyte and ld to d
	ld a,b:srl a:add e:ld e,a
ret


conv_coords2tilepos:
	;in b,c (x,y coords) - out b,c (x,y tilepos)
	srl b:srl b
	srl c:srl c
	ret


conv_mapaddr2scraddr:
	;input de=mapaddress - via bc
	ld a,e:and 31
	ld b,a				;b=e mod 32
	ld a,d:sub MAP_HI
	;al=c*32+b
	rl e:adc a
	rl e:adc a
	rl e:adc a
	ld c,a				;c=de \ 32

conv_tilepos2scraddr:
	ld a,WID_TILES:cp b
error1:
jr c,$					;If this still hapens after publishing, I'm in trouble
ld a,HGT_TILES:cp c
error2:
jr c,$


	;input b=x, c=y TILE coords
	;output de=scraddr
	;de=_lut_scraddr(c*8)+(b/2)+(S_curr_screen_org)*#100
	ld h,_lut_scraddr/#100
	ld a,c:add a:add a:add a:ld l,a
	ld e,(hl):inc l
	ld a,b:rla:add e:ld e,a
	ld a,(S_curr_screen_org)
	adc (hl):ld d,a
ret

conv_coords2mapaddr:
	;in b,c - out hl
	srl b:srl b
	srl c:srl c
conv_tilepos2mapaddr:
	;b=x, c=y pos in stagemap
	;hl=address in stagemap (+c*32+b)
	xor a:ld l,a			;l=0 and reset cf
	ld a,c
	rra:rr l				;shift right x3 = *32
	rra:rr l
	rra:rr l
	add MAP_HI:ld h,a
	ld a,l:add b:ld l,a
ret


conv_itemid2gfxaddr:
	add TILE_ITEM_MIN
	cp TILE_ITEM_MIN-1
	jr nz,$+3
		xor a

conv_tileidx2gfxaddr_base:
	ld l,TILESET_OFFSET_40

conv_tileidx2gfxaddr:
	;l must already contain tileset offset
	cp TILE_COMB_MIN
	jr nc,conv_tileidx2gfxaddr_combination_tile
	or a
	rra:rr l
	rra:rr l
	rra:rr l
	add _tiles/#100
	ld h,a
	res 4,h
	bit 4,a
	ret

conv_tileidx2gfxaddr_combination_tile:


	;bit 0-1 -> bit 4-5 in l
	;bit 2-3 -> 4-5 in hibyte
	sub TILE_COMB_MIN

	;00 - #C6C0
	;01 - #C6E0
	;02 - #CEC0
	;03 - #CEE0
	rl l
	adc a
	add a		;2-4 -> 3-5
	ld h,a

	add a:add a:add a	;1-2 -> 4-5
	and %00110000
	add #C0
	ld l,a

	ld a,%00111000
	and h
	add #C6
	ld h,a
	xor a
	ret



; COLLISION
;-----------

check_collision:
;also checks if protecting potion active
;returns nz if collision
	;don't check during dying
	ld a,(S_pau_dies_countdown)
	or a:ret nz
	;first, collision with enemies
	ld b,MAX_ENEMIES_PER_STAGE
	ld iy,_enemies_curr_stage		;points to curr coords
		check_enemies_coll_lp:
		ld a,(iy+ENE_SPEED):dec a
		jp m,check_collision_next	;if 0, -1 or -2, don't check
		ld d,(iy+ENE_CURR_MOV)
		ld e,(iy+ENE_FIX_COORD):inc hl
		bit 7,(iy+ENE_SPRITE)		;get sprite orientation
		ld hl,#0604
		jr z,$+7					;if hori mvt
			ld a,e:ld e,d:ld d,a	;exchange x&y
			ld h,8					;and make wider
		;collision detection
		ld a,(pau_x_coord)
		sub d
		jr nc,$+6
			neg:ld h,PAU_WIDTH-1
		sub h
		jp p,check_collision_next
		;y:
		ld h,a						;backup y for later
		ld a,(pau_y_coord)
		sub e
		jr nc,$+6
			neg:ld l,PAU_HEIGHT-2
		sub l
		jp p,check_collision_next
		;check if paulina is on top of enemy
		cp #fe
		jr c,enemy_hits_pau
		;nc
	S_pau_falling:
		nop							;or scf if she's falling
		jr c,enemies_killed			;she hits.
		enemy_hits_pau:
		ld a,#80					;set bit 7 in collision marker
		jr check_collision_animated
		check_collision_next:
		ld de,LGTH_ENEMY_DATA
		add iy,de
		djnz check_enemies_coll_lp
		xor a
	check_collision_animated:
	;then collision with animated tiles:
	S_collision_animated_tile equ $+1
		or SMC_PLACEHOLDER			;or with rest of collision marker
	S_collision_protection equ $+1
		and 0; #ff


	jp nz,pau_looses_life


	ret

enemies_killed:
	ld a,SFX_KILL
	call snd_playsfx
	ld (iy+ENE_SPEED),-1			;set enemy to fall

	ld a,(iy+ENE_FIX_COORD)
	ld c,(iy+ENE_SPRITE)
	sla c							;x2 and get bit 7
	jr c,enemies_killed_align		;if not already vertical movement:
		set 7,(iy+ENE_SPRITE)		;set to vertical movement
		;exchange coords
		ld b,a
		ld a,(iy+ENE_CURR_MOV)
		ld (iy+ENE_CURR_MOV),b
		ld (iy+ENE_OLD_MOV),b
	enemies_killed_align:
	inc a:inc a:and #FC
	ld (iy+ENE_FIX_COORD),a			;align to tile coord (for correct erasing)
	ld a,c							;get enemy number *2
	add a:add a:add a				;shift enemy number to top nibble
	ld c,a
	ld hl,(this_stages_enemies)
		enemies_killed_search_it_in_list:
		ld a,(hl)
		and #F0
		cp c
		inc hl:inc hl:inc hl
		jr nz,enemies_killed_search_it_in_list
	dec hl						;go back to 3nd item in list
	res 7,(hl)					;reset alive bit

ret

enemies_kill_all:
	ld iy,_enemies_curr_stage
	ld b,MAX_ENEMIES_PER_STAGE
		enemies_kill_all_loop:
		push bc
		xor a:cp (iy)
		call nz,enemies_killed
		ld de,LGTH_ENEMY_DATA
		add iy,de
		pop bc
		djnz enemies_kill_all_loop
ret


check_tiles_behind_paulina_bottom_row:
	;check only the bottom row
	Add_to_reg 9,c						;bottom pixel line of paulina
	ld e,1								;only one row to examine
	jr check_tiles_behind_paulina_later

check_tiles_behind_paulina:
	;input bc=coords
	;output e=amount of blocking tiles
	;		d=amount of ladder tiles

	ld e,3								;constant
	ld a,c:and e
	cp e:jr c,$+3
		inc e							;3 lines or 4 if y mod 4 > 3

check_tiles_behind_paulina_later:
	inc b								;leave 2 px margin for collision
	call conv_coords2mapaddr
	ld b,e								;line counter
	ld de,0								;tile counters
	check_tile_collisions_lo:
		ld c,0
		call check_tile_row
		ld a,WID_TILES-2
		Add_A_to_HL
		djnz check_tile_collisions_lo
	ret

	check_tile_row:
	;input hl=map address
	;checks 2 adjacent tiles,
	;increases e for each blocking tile and d for each ladder
	call $+3				;do this twice

	ld a,(hl)
	and %01111111			;remove mirror bit
	cp TILE_MAX_LADDER+1
	jr nc,$+3
		inc d
	cp TILE_MAX_FG+1
	jr nc,$+4
		dec d
		inc e
	inc hl
	ret

; GFX MANIPULATION
;------------------


do_bomb_effect:
	ld hl,_palette40
	ld b,8
		ld (hl),#54:inc hl
		djnz $-3

	ld de,_bomb_colors
		bomb_loop:
		ld a,(de):inc de
		ld (_palette40),a
		ld (_palettec0),a
		ld (S_border_color),a
		ld b,15
			halt:djnz $-1
			or a
			jr nz,bomb_loop
	ld hl,_palette_backup
	ld de,_palette40
	ld bc,8
	ldir
	ld a,#54
	ld (S_border_color),a
ret



erase_gamescreen:
	ld b,64:ld c,160
	ld hl,#4000
	jr erase_area

erase_menu_area:
	ld hl,#4040:ld bc,#16a0
	jr erase_area


erase_potion_icon_countdown_and_res_message_bit:
	res 3,(hl)

erase_potion_icon_countdown:
	ld hl,POTION_TIMER_SCRADDR
	ld bc,#0C04
	call erase_area
erase_potion_icon:
	ld hl,POTION_ACTIVE_SYM_SCRADDR
	ld bc,#040F

erase_area:
	;input b=with (bytes), c=height (lines)
	;hl=top left origin
	push hl:push bc
	call erase_area_do
	pop bc:pop hl
	set 7,h

	erase_area_do:
	xor a:ld d,a:ld e,b
		erase_area_ol:
			ld a,b
			erase_area_il:
				ld (hl),d
				inc hl
				djnz erase_area_il
			sbc hl,de
			dec c
	ret z
			ld b,a
			call nextline_hl
			or a
			jr erase_area_ol

combine_tiles:
	;input - indices of bg, fg & new tile
	;a=new tile, b=bg tile, c=fg tile

	ld ixh,c
	;1. calculate addresses for bg tile & new tile
	call conv_tileidx2gfxaddr_base
	ex de,hl					;de = addr new tile
	ld a,b

	call conv_tileidx2gfxaddr_base		;addr bg tile

	;2. copy bg to new (mirrored if needed)
	;hl=source, de=dest
	push de
	bit 7,b:jr nz,ct_mirrored
	;tile not mirrored:
	ld bc,32:ldir
	jr ct_reset_de

	ct_mirrored:
	ld ixl,16
	ld b,_lut_mirror/#100
		ct_mirrored_lp:
		ld c,(hl):ld a,(bc):inc l
		inc e:ld (de),a
		ld c,(hl):ld a,(bc):inc l
		dec e:ld (de),a:inc e:inc e
		dec ixl:jr nz,ct_mirrored_lp

	ct_reset_de:
	pop de
	;3. calc addr of item over new tile
	ld a,ixh
	call conv_itemid2gfxaddr ;hl=fg tile
	ld b,h:ld c,l
	ld ixl,32
	ld h,_lut_transparency_masks/#100
		ct_or_loop:
		ld a,(bc):inc c
		Draw_transp_byte a:inc e
		dec ixl:jr nz,ct_or_loop
	ret


mirror_pau_if_necess:
	;returns nz if mirroring was executed
	ld hl,(S_pau_spr_addr)
	ld l,-1
	ld a,(pau_spr_orientation)
	xor (hl)
	ret z

;do mirroring
	ld a,(hl)					;invert orientation
	xor -1
	ld (hl),a
	inc l						;go to gfx
	ld b,62
	jr mirror_sprite


mirror_ene_if_necess:
	ld iy,_enemies_curr_stage
		check_ene_mirror:
		ld a,(iy+ENE_SPEED)
		or a:jr z,check_ene_mirror_next
		ld a,(iy+ENE_SPRITE)
		bit 7,a
		jr nz,check_ene_mirror_next		;dont mirror if vertical mvmt
		ld b,a							;backup
		ld hl,_sprite_orientations
		Add_A_to_HL
		ld a,(hl):ld c,(iy+ENE_DIRECTION)
		xor c
		rla								;check if same bit 7
		jr c,check_ene_mirror_next
			;this sprite needs mirroring
			ld a,(hl):cpl:ld (hl),a
			ld a,b
			add _ene_sprs/#100
			ld h,a:ld l,0:ld b,64
			call mirror_sprite
		check_ene_mirror_next:
		call enemies_iy_next_in_list
		jr nz,check_ene_mirror
ret


mirror_sprite:
	;hl=gfx adress, b=height
	ld d,_lut_mirror/#100
	mirror_spr_line:
		ld e,(hl)			;get (3)
		ld a,(de):ld c,a	;get 3
		inc l:ld e,(hl)		;get (2)
		ld a,(de)			;get 2
		inc l:ld e,(hl)		;get (1)
		ld (hl),a			;write 2
		ld a,(de)			;get 1
		inc l:ld e,(hl)		;get (0)
		ld (hl),c			;write 3
		ld c,l				;backup x+3
		dec l:dec l
		ld (hl),a			;write 1
		ld a,(de)			;get 0
		dec l:ld (hl),a		;write 0
		ld l,c:inc l		;x+4 = next line
		djnz mirror_spr_line
	inc b					;reset z flag
ret

;####################
;#                  #
;# SCREEN BUFFERING #
;#                  #
;####################


screen_switch_buffer:
	ld hl,frame_counter
	inc (hl)
	ld bc,S_curr_screen_org
	ld a,(bc)
	xor #80
	ld (bc),a
	ld de,#3000+TILESET_OFFSET_40
	ld hl,_palette40
	jp p,write_screen_vals
		ld de,#1000+TILESET_OFFSET_C0
		ld hl,_palettec0
	write_screen_vals:
	ld b,#7F						;CRTC port
	xor a
	setcolors0to3_lp:
		out (c),a:outi:inc b		;out ink & value
		inc a:cp 4
		jr nz,setcolors0to3_lp
S_border_color equ $+1
	ld a,#54:ld c,#10
	out (c),c:out (c),a

	ld bc,#bc0c
	out (c),c
	inc b
	out (c),d
	ld a,(display_mode):or a
	ret z
		ld a,e						;only change conv_tileidx2gfxaddr
		ld (S_tileset_screen_offset),a		;if game is running
ret

set_mode_zero:
	xor a
	jr set_mode

set_mode_timestop:
	ld a,MODE_TIMESTOP
	jr set_mode

set_mode_jumpnrun:
	;find out if time stop potion is active
	ld a,(S_potion_countdown)
	inc a:jp z,set_mode_jumpnrun_regular
	ld a,(curr_potion_recipe)
	cp 6:jr nz,set_mode_jumpnrun_regular
	ld a,MODE_TIMESTOP
	jr set_mode
	set_mode_jumpnrun_regular:
	ld a,MODE_JUMPNRUN
	jr set_mode

set_mode_menu:
	ld a,MODE_MENU

set_mode:
	ld (display_mode),a
	ret

; ITEMS
;-------------

;Coding in item list:

; 16 Bits
; 0-4 x
; 5-9 y
; 10-13 item type
; 14-15 status:
; 00 won't happen -> stage number
; 01 off, preset
; 10 active, dropped
; 11 active, preset

;SSTTTTYY YYYXXXXX

items_search_stage:
;search for the beginning of the current stage in the item list
;and write it to this_stages_items
;out hl & (this_stages_items)=location of first item
;    abc destroyed
	ld hl,_all_items_list-2
	ld bc,all_items_list_end-_all_items_list
	ld a,(S_curr_stage)
	items_search_stage_loop:
		inc hl
		inc hl
		cp (hl)
		jr z,items_search_stage_found
		jr c,items_search_stage_loop
		jr items_search_stage_loop+1
	items_search_stage_found:
	inc hl
	ld (this_stages_items),hl
ret


stage_init_items:
	xor a
	ld (amount_items_in_stage),a
	;delete combination tiles list
	ld b,MAX_ITEMS_PER_STAGE
	ld hl,_comb_tiles
		ld (hl),-1:inc hl:djnz $-3
	call items_search_stage

	items_go_through_this_stage:
		ld a,(hl)
		and 3						;isolate top 2 bits of y coord
		add MAP_HI:ld d,a			;convert to map address hibyte
		ld a,(hl):inc hl			;reload hibyte of info word
		ld e,(hl):inc hl			;de=map address
		cp #40
	ret c							;bits 6&7=0 -> eol
		cp #80
	jr c,items_go_through_this_stage	;bit 7=0 -> deactivated
		rra
		rra							;bit 7=1, -> item is active
		and #F						;a=item type
		call items_add_to_stagemap
		jr items_go_through_this_stage


items_pick_up_item:
	res 0,(hl)					;reset bit in message byte
	;remove item from list & map
S_item_behind_pau equ $+1
	ld de,SMC_PLACEHOLDER		;address in stagemap
	xor a:cp d
ret z						;no item behind her
items_pick_up_item_entry_for_collect:
	;1. check if inventory full (ret if so)
	ld a,(inv_curr_amount)
	cp INVENTORY_SIZE
ret z
	ld a,SFX_PICKUP
	call snd_playsfx
	;get bg item
	ld a,(de)				;comb nr (+TILE_COMB_MIN)
	ld hl,_comb_tiles-TILE_COMB_MIN
	Add_A_to_HL
	ld a,(hl)				;bg tile
	ld (de),a				;write old bg to stagemap
	ld (hl),-1				;mark comb slot as free
	call redraw_tile
	ld hl,wait_frame_after_picking
	inc (hl)
	ld hl,amount_items_in_stage
	dec (hl)

	;search the item @ coords de in the _all_items_list
	;deactivate it and return item type in a
	ld a,d
	sub MAP_HI
	ld d,a
	ld hl,(this_stages_items)
	items_search_coords_loop:
		ld a,(hl)   ;highbyte
		inc hl
		bit 7,a     ;skip if deactivated
		jr z,items_search_coords_next
		sub d
		and 3       ;isolate
		jr nz,items_search_coords_next
		ld a,(hl)   ;lowbyte
		cp e
		jr z,items_pick_up_this      	 	;found
		items_search_coords_next:
		inc hl      ;next item
		jr items_search_coords_loop


	items_pick_up_this:
	call items_deactivate_item
	items_add_to_inventory:
	rra:rra:and #F  ;isolate type
	ld d,a
	ld hl,inv_curr_amount
	ld a,(hl)
	inc (hl)
	add _inventory:ld c,a
	ld b,_inventory/#100
	jr nc,$+3
		inc b
	ld a,d
	ld (bc),a				;add item to inventory
	jp draw_inventory


	items_deactivate_item:
	;deactivate (if preset) or remove item in list, return item type
	dec hl			;go back to highbyte
	ld a,(hl)		;save hibyte (for later)
	bit 6,a     	;check if preset
	jr nz,items_deactivate_preset
	items_remove_item:
	;don't deactivate, instead delete it from list
	;de=x,hl=x+2,bc=end-(x+2),ldir
	call items_calc_shift_bc
	inc bc:inc bc
	ld h,d:ld l,e
	inc hl:inc hl
	ldir
	ret

	items_deactivate_preset:
	res 7,(hl)      ;doesnt hurt if after ldir
	ret


items_drop_from_inv:
	;b=column, c=row-3
	ld a,b
	inc c
	jr nz,$+4
		add 6
	ld e,a						;e=position of item in inventory to drop
	ld hl,_inventory			;calc addres in inventory
	add hl,de
	ld a,(hl):ld ixl,a			;get item number
	inc a
ret z							;if nothing on this position
	push de:push hl

	ld hl,(pau_coords)
	ld bc,#0308:add hl,bc		;move coords to bottom middle of pau
	ld b,h:ld c,l
	call conv_coords2mapaddr

	call items_drop_one
	pop hl:pop de

	ld a,SFX_BUZZER
	jp z,snd_playsfx			;if not successful, buzz & cancel

	call items_remove_from_inv
	ld a,SFX_DROP
	call snd_playsfx
	jp set_mode_jumpnrun


items_remove_from_inv:
	;e=item index, hl=adress in _inventory
	ld a,INVENTORY_SIZE:sub e	;calc length to shift
	ld c,a:ld b,0
	ld d,h:ld e,l				;dest cell
	inc hl						;source cell
	ldir
	ld hl,inv_curr_amount
	dec (hl)
	call draw_inventory

ret

items_drop_one:
	;hl adress in tilemap, ixl tile type
	;returns nz if successful
	;check if room
	ld a,(amount_items_in_stage)
	cp #10
ret z
;check if not already combtile here
	ld a,(hl)
	sub TILE_COMB_MIN:cp #10
	jr nc,items_drop_it
xor a:ret
	items_drop_it:
	push hl						;for adding to stagemap later
	ex de,hl					;get address in tilemap
	ld a,ixl
	;fall through

;items_insert_item:
;make room for a new item in this stage's list and insert it
;in  de=coords (or address) & a=type
;out hl=conserved,bc=1,de=item word
	;calculate item info word
	add a:add a
	ld b,a
	ld a,d:sub MAP_HI
	and 3			;isolate top 2 bits of y
	or b
	set 7,a
	ld d,a
	push de
	;shift items to make room
	ld hl,(this_stages_items)
	;de=end-1,hl=end-3,bc=end-(x+2)
	call items_calc_shift_bc
	ld hl,all_items_list_end-3
	ld de,all_items_list_end-1
	lddr
	ex hl,de    ;addr+1
	;insert item info word:
	pop de
	ld (hl),e:dec hl:ld (hl),d
	;fall through
	pop de:ld a,ixl

items_add_to_stagemap: ;and comb list
	;de=coords in map, a=item type
	push hl
	ld c,a
	;look for an unused combination tile

	;ersetzen mit cpdr
	ld b,MAX_ITEMS_PER_STAGE
	ld hl,_comb_tiles
	ais_look_combt_lp:
		ld a,(hl):inc a		;check if -1
		jr z,comb_tile_found
		inc hl
		djnz ais_look_combt_lp

	comb_tile_found:
	ex de,hl					;de<-comb list, hl<-coords
	ld a,MAX_ITEMS_PER_STAGE
	sub b:add TILE_COMB_MIN
	ld b,(hl)					;get bg item
	ld (hl),a					;save comb item
	ex de,hl
	ld (hl),b					;save bg item (-1!) in comb list
	;a=new, b=bg, c=fg
	call combine_tiles
	ld hl,amount_items_in_stage
	inc (hl)
	pop hl						;z is reset (inc (hl))
ret

items_calc_shift_bc:
;calculate length for ldir/lddr
;in hl=start
;out bc(&hl)=length
	ex de,hl
	ld hl,all_items_list_end-2
	or a:sbc hl,de
	ld b,h:ld c,l
ret





brew_potion:
	;reset missing ingredients counter
	xor a:ld (S_bp_missing_ing),a
	;calc ingredient list of recipe
	ld a,(curr_potion_recipe)
	add a:add a:add a
ret m
ret z						;if first or last page
	ld de,_all_potions_recipes-8
	Add_A_to_DE
	ld ixl,6
		bp_go_through_ing_lp:
		ld a,(de):or a
		call p,bp_check_ingredient
		inc de
		dec ixl:jr nz,bp_go_through_ing_lp
S_bp_missing_ing equ $+1
	ld a,SMC_PLACEHOLDER
	or a
	jr nz,bp_not_enough_ingredients
	ex hl,de
	ld e,(hl):inc hl:ld d,(hl)
	push de
	;set message that this potion is active
	ld hl,message_byte
	set 1,(hl)
	;remove items from inventory, from top to bottom
	ld hl,_inventory
	ld e,0
	bp_remove_ingredients_lp:
		ld a,(hl):cp -1
		jr z,potion_activate		;end loop if -1
		rlca:jr nc,bp_dont_remove	;check if bit 7 set
		push hl:push de
		call c,items_remove_from_inv
		pop de:pop hl:dec hl
		bp_dont_remove
		inc hl:inc e
		jr bp_remove_ingredients_lp
	potion_activate:
	ld a,(curr_potion_recipe)
	dec a
	ld de,POTION_ACTIVE_SYM_SCRADDR
	call draw_potion_symbol
	ld a,SFX_BREW
	call snd_playsfx
	ld b,100:halt:halt:djnz $-2
	jr bp_remove_missing_marker_bits
	;(when finished, reads activation sub address from stack)

bp_not_enough_ingredients:
	ld a,SFX_BUZZER:call snd_playsfx
bp_remove_missing_marker_bits:
	ld hl,_inventory
		bp_not_enough_ingredients_lp
		 ld a,(hl)
		 inc a
		 ret z
		 res 7,(hl)
		 inc hl
		 jr bp_not_enough_ingredients_lp
		ret

bp_check_ingredient:
	;a=ingredient number
	ld hl,_inventory
	ld bc,INVENTORY_SIZE
	cpir:dec hl
	jr nz,bp_mark_missing
	;set bit 7 of ingredients that are going to be added
	set 7,(hl)
	ret

bp_mark_missing:
	ld hl,S_bp_missing_ing
	inc (hl)
	ld a,6:sub ixl
	add a:add _book_item_syms:ld l,a
	ld h,_book_item_syms/#100
	jr nc,$+3
		inc h
	ld a,(hl):inc hl
	ld h,(hl)
	ld l,a
	push hl
	call bp_cross
	pop hl
	set 7,h

bp_cross:
	ld c,%11001100
	ld b,8
		bp_cross_lp:
		ld a,c:and (hl):ld (hl),a:inc hl
		ld a,c:and (hl):ld (hl),a:dec hl
		rlc c
		call nextline_hl
		djnz bp_cross_lp
	ret

potion_check_countdown:
S_potion_countdown equ $+1
	ld a,SMC_PLACEHOLDER
	or a
ret m
	jp nz,update_potion_timer
	jp potion_stop_impact


potion_walk_through_walls:
	xor a
	ld (S_potion_jump_walking_walls),a
	jr potion_start_countdown

potion_jump_high:
	ld a,JUMP_STEPS_BIGJUMP
	ld (S_jump_height),a
	jr potion_start_countdown

potion_stop_time:
	call set_mode_timestop
	jr potion_start_countdown

potion_water_protect:
	ld a,#fe
	jr potion_write_collision_protection

potion_fire_protect:
	ld a,#fd
	jr potion_write_collision_protection

potion_gas_protect:
	ld a,#fb
	jr potion_write_collision_protection

potion_waves_protect:
	ld a,#f7
	jr potion_write_collision_protection

potion_enemy_protect:
	ld a,#7f

potion_write_collision_protection:
	ld (S_collision_protection),a

potion_start_countdown:
	ld a,POTION_DURATION
	ld (S_potion_countdown),a
	call draw_potion_timer
	jp potion_return_to_game

;potions without timer
potion_home:
	call set_mode_zero
	ld a,STAGE_HOME
	jr potion_teleport

potion_random_teleport:
	call set_mode_zero
	call get_random_byte
	and 31
	ld hl,S_curr_stage
	cp (hl)
	jr z,potion_random_teleport
potion_teleport:
	push af
	ld a,-1
	ld (S_pau_coords_in_new_stage),a
	call potion_erase_screen
	pop af
	ld (S_curr_stage),a
	ld hl,_stages_layout
	ld bc,0
	cpir
	dec hl
	ld (S_curr_stage_position),hl
	ld a,SFX_ARPEGGIO
	call snd_playsfx

	jp gameloop_stage_update

potion_3_random_items:
	ld b,3
	potion_3_rand_loop:
	push bc
	ld b,100:halt:djnz $-1
	call get_random_byte
	and 15
	ld i,a
	call items_add_to_inventory
	ld a,SFX_PICKUP
	call snd_playsfx
	pop bc
	djnz potion_3_rand_loop
	jr shortcut_e_c_a_r

potion_1up:
	;missing:
	;do some cool stuff
	ld b,130:halt:djnz $-1
	ld a,(S_pau_lives)
	inc a
	cp 10
jp z,shortcut_e_c_a_r
	ld (S_pau_lives),a
	ld a,SFX_ARPEGGIO
	call snd_playsfx
	call draw_lives_display
	jr shortcut_e_c_a_r

potion_bomb:
	call set_mode_timestop
	ld a,SFX_BOMB
	call snd_playsfx
	call do_bomb_effect
	call enemies_kill_all
	call draw_stagemap
shortcut_e_c_a_r:
	jp potion_erase_countdown_and_return

potion_collect:
	ld de,_stagemap
	ld bc,#280
		potion_collect_search_loop:
		push bc:push de
		ld a,(de)
		cp #F0
		jr nc,potion_collect_item_found
		or #81:cp #D7
		jr nz,potion_collect_no_item
		;gem found:
			ld a,#40:ld (de),a
			call add_one_gem
			ld a,SFX_GEM
			call snd_playsfx
			jr potion_collect_erase_on_screen
		potion_collect_item_found:
		call items_pick_up_item_entry_for_collect
		potion_collect_erase_on_screen:
		pop de:push de
		call redraw_tile
		call wait_a_moment

		potion_collect_no_item:
		pop de:pop bc
		inc de:dec bc
		ld a,b:or c
		jr nz,potion_collect_search_loop
	call draw_stagemap
	jr potion_erase_countdown_and_return


potion_double_gems:
	ld de,_gems_in_pocket+2
	ld bc,#300
		potion_double_gems_lp:
		ld a,(de)
		add a:add c
		ld c,0
		cp 10
		jr c,potion_double_gems_no_carry
			sub 10
			ld c,1
		potion_double_gems_no_carry:
		ld (de),a:dec de
		push bc:push de
		call draw_gems_display
		ld a,SFX_GEM
		call snd_playsfx
		call wait_a_moment
		pop de:pop bc
		djnz potion_double_gems_lp

	jr potion_erase_countdown_and_return


potion_disappear:
	ld a,SFX_ARPEGGIO
	call snd_playsfx
	;makes all animated tiles (water, fire, gas & GEMS) disappear
	ld hl,_animated_tiles
potion_disappear_search_loop:
	ld b,(hl):inc hl:ld c,(hl)
	inc b
jr z,potion_erase_countdown_and_return
	push hl
	dec b
	call conv_tilepos2scraddr
	call conv_tilepos2mapaddr
	ld a,#40
	ld (hl),a
	call draw_tile_a_to_both_screens_de
	pop hl
	inc hl
	jr potion_disappear_search_loop

potion_erase_countdown_and_return:
	call erase_potion_icon_countdown
potion_return_to_game:
	jp set_mode_jumpnrun


potion_stop_impact:
	;protection:
	ld a,#ff
	ld (S_collision_protection),a
	;reset countdown
	ld (S_potion_countdown),a
	;jump:
	ld a,JUMP_STEPS_DEFAULT
	ld (S_jump_height),a
	;walking through walls
	ld a,OP_CPE
	ld (S_potion_jump_walking_walls),a
	jr potion_erase_countdown_and_return

potion_erase_screen:
	call set_mode_zero
	call erase_gamescreen
	jp erase_potion_icon

potion_countup:
	ld a,(frame_counter)
	and 3							;every 4th frame, update skip countdown
ret nz
	ld hl,S_potion_countdown
	ld b,(hl)
	dec b
ret m						;if it was 0 or negative, don't update
	ld (hl),b
	ld c,a
	ld a,#7
	and b
	jr nz,$+7
		ld a,SFX_BEEP
		call snd_playsfx
	ld a,c
ret


;####################
;#                  #
;# STAGEMAP & TILES #
;#                  #
;####################


stage_unpack:
	;search and save position in gem list
	ld a,(S_curr_stage)
	ld hl,_all_gems-1
	or a
jr z,gem_list_save
	add a:add a:dec a
		gem_list_pos_lp:
		inc hl:inc hl
		cp (hl)		;if a<hibyte, that's where this stage starts
		jr nc,gem_list_pos_lp
	gem_list_save:
	dec hl
	ld (S_curr_stages_gems),hl

	ld hl,_packed_stagemaps
	ld d,0
S_curr_stage equ $+1
	ld a,SMC_PLACEHOLDER
	or a
	jr z,stage_unpack_save_frequent
	stage_unpack_search_stage:
		ld e,(hl)
		add hl,de
		inc hl
	dec a:jr nz,stage_unpack_search_stage

	stage_unpack_save_frequent:
	ld a,(message_byte):bit 4,a
	res 4,a:ld (message_byte),a
	inc hl:ld a,(hl)
	call snd_play_song
	inc hl:ld b,(hl)
	inc hl:ld c,(hl)
	ld a,(S_pau_coords_in_new_stage)
	inc a
	jr nz,$+6
		ld (S_pau_coords_in_new_stage),bc
	ld (_frequent_tiles),hl
	ld e,8
	add hl,de

    push hl
    pop ix ;begin packed map
    ld iyl,1 ;init for bits
    ld de,_stagemap
	unpsm_lp:
	ld b,2
	call get_bitfrag				;get compression type marker
; 00 normal    +7
; 01 freq      +3
; 10 repeat    +4
; 11 pattern   +6 +4
	or a:jr nz,unpack_check_freq
	;normal
	ld b,7:call get_bitfrag
	jr unpack_check_tilechange
	unpack_check_freq:
	dec a:jr nz,unpack_check_repeat
	;frequent
	ld b,3:call get_bitfrag
	or a
	jr nz,unpack_freq_not_0
		ld a,TILE_BG_BLACK
		jr unpack_add_tile
	unpack_freq_not_0:
_frequent_tiles equ $+1
	ld hl,SMC_PLACEHOLDER
	Add_A_to_HL
	ld a,(hl)
	unpack_check_tilechange:
	;transcode to 8bit values

	cp #50
	jr c,$+4
		xor #c0
	unpack_add_tile:
	ld (de),a
	inc de
	jr unpsm_lp
	unpack_check_repeat:
	dec a:jr nz,unpack_pattern
	;repeated tile
	ld b,4:call get_bitfrag
	ld h,d:ld l,e
	dec hl
	push bc
	add 2
	ld c,a:ld b,0
	ldir
	pop bc
	jr unpsm_lp
	unpack_pattern:
	ld b,6:call get_bitfrag
	cpl:ld l,a:ld h,#ff
	add hl,de
	ld b,4:call get_bitfrag
	add 3
	push bc
	ld c,a:ld b,0
	ldir
	pop bc
	;dec de
	ld hl,stagemap_end
	or a:sbc hl,de
	jr nc,unpsm_lp
	ld b,32:ld hl,stagemap_end
	ld (hl),#40:inc hl:djnz $-3
	ret






get_bitfrag:
    ;input b=amount of bits
    ;local - do not touch:
    ;iyl=bitpair counter
    ;c=current byte
    xor a
    get_bp_lp:
    dec iyl:call z,getnextbyte
    rl c:rla
    djnz get_bp_lp
    ret

getnextbyte:
    ld c,(ix):inc ix
    ld iyl,8
    ret


stage_add_gems:
	ld a,(S_curr_stage)
	add a:add a				;*4 for finding correct hibyte
	ld b,a					;backup
	ld hl,_all_gems			;hibyte
		stage_add_gems_lp:
		ld e,(hl):inc hl
		ld a,(hl):inc hl
		cp #ff
ret z
		xor b
		cp 4
		jr nc,stage_add_gems_lp
		add _stagemap/#100:ld d,a
		ld a,#bf
		ld (de),a
		jr stage_add_gems_lp



; RANDOM NUMBERS
;----------------

get_random_byte:
S_get_random_byte_var EQU $+1
    ld a,51 ; Seed
    add a,a
    jr nc,_get_random_byte
    xor %00011101
_get_random_byte:
	ld (S_get_random_byte_var),a
    ; crapification
    ld c,a
    ld a,r
    add a,c
ret

;***********
;* ENEMIES *
;***********

stage_init_enemies:
	;erase old data
	ld b,LGTH_ENEMY_DATA*MAX_ENEMIES_PER_STAGE
	ld hl,_enemies_curr_stage
	;THIS COULD ACTUALLY BE A SUBROUTINE
		xor a:ld (hl),a:inc hl:djnz $-2

	;search stage in compressed list
	ld a,(S_curr_stage)
	ld hl,_all_enemies_list+1
	or a:jr z,enemies_save_this_stage_address

	ld hl,_all_enemies_list-3				;3 bytes per enemy in list
	ld c,a:ld de,3
		enemies_init_search_stage_marker:
		add hl,de
		ld a,(hl)
		sub #E0
		jr c,enemies_init_search_stage_marker
		inc hl
		cp c
		jr nz,enemies_init_search_stage_marker+1
	enemies_save_this_stage_address:
	ld (this_stages_enemies),hl

	;unpack enemy info
	enemies_init_unpack:
	ex de,hl
	ld iy,_enemies_curr_stage
	enemies_unpack_loop:
		ld a,(de)
		cp #E0
	ret nc
		and 3:inc a:ld b,a
		ld a,(de)
		rrca:rrca:rrca
		ld (iy+ENE_DIRECTION),1	;default
		jr c,$+6
			ld (iy+ENE_DIRECTION),-1
		rrca:and %10001111
		ld (iy+ENE_SPRITE),a

		inc de
		ld a,(de):ld c,a
		and #1F
		add a:add a					;*4
		ld (iy+ENE_FIX_COORD),a
		inc de

		ld a,(de):inc de
		bit 7,a:jr nz,$+4
			ld b,0					;set speed to 0 if dead
		ld (iy+ENE_SPEED),b
		ld (iy+ENE_COUNTER),b
		rra:rr c
		rra:rr c
		and #1F:add a:add a			;min*4
		inc a
		ld (iy+ENE_MIN_MOV),a
		ld a,c
		rra:and %01111100:dec a
		ld (iy+ENE_MAX_MOV),a

		bit 7,(iy+ENE_DIRECTION)
		ld a,(iy+ENE_MIN_MOV)
		jr z,$+5					;if negative direction
			ld a,(iy+ENE_MAX_MOV)	;start with max value
		ld (iy+ENE_CURR_MOV),a

		ld bc,LGTH_ENEMY_DATA
		add iy,bc
		jr enemies_unpack_loop
	ret


enemies_update:
	ld iy,_enemies_curr_stage
	enemies_update_loop:
	call enemies_update_do
	call enemies_iy_next_in_list
	jr nz,enemies_update_loop
ret

enemies_update_do:
	;check if update (counter=speed)
	ld a,(iy+ENE_SPEED)
	or a:ret z					;if 0, enemy is dead
	inc a:jr z,enemies_update_fall	;if -1, enemy falls
	jp m,enemies_update_reached_ground
	dec (iy+ENE_COUNTER)
ret nz							;update only if counter=0
	ld (iy+ENE_COUNTER),a		;reset counter
	ld a,(iy+ENE_CURR_MOV)		;get x
	ld (iy+ENE_OLD_MOV),a		;copy to old

	add (iy+ENE_DIRECTION)		;add move offset
	ld (iy+ENE_CURR_MOV),a		;save new
	cp (iy+ENE_MIN_MOV)			;check min boundary
	jr c,ue_turn
	cp (iy+ENE_MAX_MOV)			;check max boundary
ret c
	ue_turn:
	xor a
	sub (iy+ENE_DIRECTION)
	ld (iy+ENE_DIRECTION),a
ret

enemies_update_fall:
	ld a,(iy+ENE_CURR_MOV)
	ld (iy+ENE_OLD_MOV),a
	ld i,a
	;check if ground below
	ld b,(iy+ENE_FIX_COORD)
	add 4:ld c,a	;tile below
	call conv_coords2mapaddr
	ld a,(hl)
	cp TILE_COMB_MIN
	jr nc,enemies_update_reached_ground
	res 7,a
	cp TILE_MIN_BG
	jr c,enemies_update_reached_ground
	;if not, write new y-coord
	ld a,i
	inc a:inc a					;fall one half tile height down
	ld (iy+ENE_CURR_MOV),a
ret

enemies_update_reached_ground:
	dec (iy+ENE_SPEED)
ret


enemies_drop_if_necess:
	;check if any enemy is @ speed -2
	ld hl,_enemies_curr_stage
	;ld b,4
	enemies_drop_check_loop:
		ld a,(hl)
		cp -5
		jr nz,enemies_drop_check_next
		ld (hl),0
		Add_to_reg ENE_SPRITE,l
		ld a,(hl)
		res 7,a:ld ixl,a
		dec hl:ld b,(hl)
		dec hl:ld c,(hl)
		call conv_coords2mapaddr
		jp items_drop_one

	enemies_drop_check_next:
		Add_to_reg LGTH_ENEMY_DATA,l
		cp LGTH_ENEMY_DATA*MAX_ENEMIES_PER_STAGE+_enemies_curr_stage
	;call enemies_iy_next_in_list
	jr nz,enemies_drop_check_loop
ret

enemies_iy_next_in_list:
	Add_to_reg LGTH_ENEMY_DATA,iyl
	cp LGTH_ENEMY_DATA*MAX_ENEMIES_PER_STAGE+_enemies_curr_stage
ret

;#############
;#           #
;# ANIMATION #
;#           #
;#############


draw_animated_tiles:

S_animated_tiles_portion equ $+1
	ld hl,_animated_tiles
	ld ixl,UPDATED_TILES_PER_FRAME
		draw_animated_tiles_loop:
		ld b,(hl):inc hl
		ld c,(hl):inc hl
		inc b:jr z,draw_animated_tiles_loop_next
		dec b
		push hl
		call conv_tilepos2scraddr
		call conv_tilepos2mapaddr
		ld a,#40:cp (hl)
		jr z,draw_animated_tiles_loop_next-1
		call get_random_byte
		and #81:xor (hl):ld (hl),a
		call draw_tile_a_to_both_screens_de
		pop hl
		draw_animated_tiles_loop_next:
		dec ixl
		jr nz,draw_animated_tiles_loop
	S_animated_tiles_frame_counter equ $+1
		ld a,AMT_FRAMES_PER_TILE_CYCLE
		dec a
		jr nz,draw_animated_tiles_save_hl
			ld a,AMT_FRAMES_PER_TILE_CYCLE
			ld hl,_animated_tiles
		draw_animated_tiles_save_hl:
		ld (S_animated_tiles_frame_counter),a
		ld (S_animated_tiles_portion),hl
		ret


draw_tile_both_screens_pos_de:
	ld a,(de):ld i,a
	call conv_mapaddr2scraddr
	ld a,i
draw_tile_a_to_both_screens_de:
	;input a=tile, de=scraddr
	ld i,a:push de:res 7,d
	ld l,TILESET_OFFSET_40
	call draw_tile_opaque_manual_screen
	pop de:ld a,i:set 7,d
	ld l,TILESET_OFFSET_C0
	jp draw_tile_opaque_manual_screen



stage_make_animated_list:
	ld hl,_animated_tiles
	ld de,stagemap_end-1
	ld ixl,TOTAL_ANIMATED_TILES
	ld c,HGT_TILES-1
	make_anim_t_l_lpo:
		ld b,WID_TILES-1
		make_anim_t_l_lpi:
			ld a,(de)
			and #3E
			cp #3E
			jr nz,make_anim_t_l_next
				ld a,(de)
				and #95
				rlca:add a
				ld ixh,a
				ld a,r:rrca					;random frame by assigning
				and #81:xor ixh				;random bits 0 & 7 to base
				ld (de),a
				ld (hl),b:inc hl			;save coords in list
				ld (hl),c:inc hl
				dec ixl						;safety check -	if list has max.
				jr z,make_anim_t_l_end		;length, ignore further tiles
			make_anim_t_l_next:
			dec de
			dec b:jp p,make_anim_t_l_lpi
		dec c:jp p,make_anim_t_l_lpo
	make_anim_t_l_end:
	;fill rest with #ff
	ld a,ixl:add a
		make_anim_fill:
		ld (hl),#ff								;eol marker
		inc hl
		dec a:jr nz,make_anim_fill
	ret

potion_disappear_search:


;####################
;#                  #
;# INPUT PROCESSING #
;#                  #
;####################


input_wait_key_after_no_key:
	call input_wait_no_key
	;fall through

input_wait_key:
	halt
	ld a,(player_input):or a			;wait until a key is pressed
	jr z,input_wait_key
	ret

input_wait_no_key:
	halt
	ld a,(player_input):or a			;wait until no key is pressed
	jr nz,input_wait_no_key
	ret

input_read_active_device:
;reads keys defined in input_keymap
;returns them in a & d, stored like in the joystick line, but inverted

	call input_initialize_read
S_active_input_device equ $+1
	ld hl,_keymap_CPC				;by default, set keyboard
	ld de,#0007						;d=output byte, e=counter
	input_read_next_key:
		call input_read_key_value
		jr z,$+3:scf				;if key is pressed, it's 1...
		rr d						;shift into d
		dec e						;count down the 7 bits
		jr nz,input_read_next_key
	ld a,d:rrca				;copy to a and complete full shift
	;fall through

input_close_read:

	ld bc,#F782:out (c),c			;close port
	ld bc,#F600:out (c),c			;(reset write)
ret

input_read_key_value:
;returns nz if this key is pressed
	ld b,#F7:outi 					;set kb line
	ld b,#F4:in a,(c)				;read value
	cpl:and (hl):inc hl				;and with required bit
	ret

input_initialize_read:
	ld bc,#F40E:out (c),c
	ld b,#B6:in a,(c)
	and #30:ld c,a
	or #C0:out (c),a:out (c),c
	inc b:ld a,#92:out (c),a
ret

wait_a_moment:
	ld b,70
	halt:djnz $-1
	ret

;####################
;#                  #
;#  INITIALISATION  #
;#                  #
;####################


initialize:
	di
	ld b,25
		flickerbuf:
		push bc
		call screen_switch_buffer
		pop bc:push bc
		ld h,b:inc h
			waiti:
			call wait_frame_flyback
			ld b,0:nop:djnz $-1
			dec h:jr nz,waiti
		pop bc
		djnz flickerbuf


	ld hl,SFX_SOUNDEFFECTS   		 ;Address of the sound effects.
	call PLY_AKM_INITSOUNDEFFECTS
	ld a,SONG_MENU
	call snd_play_song
	call set_mode_zero
	call install_interrupt_handler
	ld hl,_press_space
	call draw_string_read_color_and_scraddr
	call input_wait_key_after_no_key

	ld a,2
	ld hl,0
		erasescreens:
		ld bc,#3FFF
		add hl,bc:inc hl
		ld (hl),0
		ld d,h:ld e,l:inc de
		ldir
		dec a
		jr nz,erasescreens

	;set screen
	ld bc,#bc01:ld a,SCREEN_TOTAL_WIDTH/2
	out (c),c:inc b:out (c),a:dec b
	inc c:ld a,#2f
	out (c),c:inc b:out (c),a:dec b
	ld c,6:ld a,HGT_TILES
	out (c),c:inc b:out (c),a:dec b
	inc c:ld a,28
	out (c),c:inc b:out (c),a:dec b


;write _lut_mirror
	ld hl,_lut_mirror
		mirror_loop:
		ld a,l:and %10001000	;A
		rrca:rrca:rrca:ld (hl),a
		ld a,l:and %01000100	;B
		rrca:or (hl):ld (hl),a
		ld a,l:and %00100010	;C
		rlca:or (hl):ld (hl),a
		ld a,l:and %00010001	;D
		rlca:rlca:rlca:or (hl):ld (hl),a
		inc l
		jr nz,mirror_loop



;write_transparency table
ld h,_lut_transparency_masks/#100
	wtt_loop:
		ld a,l:cpl
		ld (hl),a
		rlca:rlca:rlca:rlca
		and (hl)
		ld (hl),a
		inc l
	jr nz,wtt_loop

;write_lut_scraddr:
ld d,l:ld e,l			;offset value to #c0 or #40
ld h,_lut_scraddr/#100
ld b,HGT_TILES:ld c,16
ld a,d
	write_lut_scraddr_lp:
	ld (hl),e:inc hl
	ld (hl),a:inc hl
	add c:bit 6,a:jr z,write_lut_scraddr_lp
	ld a,SCREEN_TOTAL_WIDTH
	add e:ld e,a
	ld a,d:adc 0:ld d,a
	djnz write_lut_scraddr_lp

ret

install_interrupt_handler:
	di
	call wait_frame_flyback
	djnz $							;wait a moment for ideal CRT beam position
	ld a,#c3
	ld (#0038),a
	ld hl,interrupt_handler
	ld (#0039),hl					;save jp address
	ei
ret

wait_frame_flyback:
		ld b,#F5
		in a,(c)
		rra
		jr nc,wait_frame_flyback
	ret



reset_game_variables:
	;reset lives
	ld a,7:ld (S_pau_lives),a

	;reset variables
	ld hl,_variables:ld b,variables_length:xor a
		ld (hl),a:djnz $-1
	dec a
	ld hl,_comb_tiles:ld b,16
		ld (hl),a:djnz $-1

	;reset gems
	ld hl,_all_gems
	reset_gems_loop:
		inc hl
		ld a,(hl)
		inc a
		jr z,reset_inventory
		res 7,(hl)
		inc hl
		jr reset_gems_loop

	reset_inventory:
		ld hl,_inventory_startup
		ld de,_inventory
		ld bc,14
		ldir

	;mark all emenies as alive
	ld hl,_all_enemies_list+1
	reset_enemies_lp:
		ld a,(hl)
		inc a
		jr z,reset_items
		cp #E1
		inc hl
		jr nc,reset_enemies_lp
		inc hl
		set 7,(hl)
		inc hl
		inc a
		jr nz,reset_enemies_lp

	reset_items:
		; 00 won't happen -> stage number
		; 01 off, preset			- set on
		; 10 active, dropped		- delete
		; 11 active, preset			- leave
		ld hl,_all_items_list
		reset_items_lp:
			ld bc,all_items_list_end
			or a:sbc hl,bc:add hl,bc
	ret nc

		bit 6,(hl)					;if default item
		jr nz,reset_def_item

		bit 7,(hl)					;dropped active item
		jr z,reset_items_skip_stagemarker	;if b7 also 0, it's the stage marker

		push hl
		call items_deactivate_item+1	;don't dec hl, it's already there
		pop hl
		jr reset_items_lp

		reset_def_item:
		set 7,(hl)

		reset_items_lp_end:
		inc hl
		reset_items_skip_stagemarker:
		inc hl
		jr reset_items_lp

ret

; SOUND

SFX_GEM equ 1
SFX_ARPEGGIO equ 2
SFX_BOMB equ 3
SFX_DIE equ 4
SFX_BREW equ 5
SFX_JUMP equ 6
SFX_DROP equ 7
SFX_PICKUP equ 8
SFX_KILL equ 9
SFX_BUZZER equ 10
SFX_QUESTION equ 13
SFX_BEEP equ 14

SONG_SILENCE equ 0
SONG_MENU equ 1
SONG_GAME equ 2

snd_playsfx:
S_sfx_toggle_code:
	nop								;or ret if sfx off
	push bc:push de:push hl
	;a=sound effect
    ld bc,#1	;channel & Volume
    call PLY_AKM_PLAYSOUNDEFFECT
	pop hl:pop de:pop bc
	ret

snd_play_song:
	ld b,a
S_music_toggle_code equ $+1
	ld a,0							;0 means on!
	or a
	ld a,b
	jr z,$+3
		xor a						;set silent song
S_snd_song_playing equ $+1
	ld b,0
	cp b							;check if this song is already playing
ret z								;return if so
	ld (S_snd_song_playing),a		;save new song
	ld (S_change_song),a
	ret

read "pau_music.asm"

;##############
;#            #
;# INTERRUPTS #
;#            #
;##############


interrupt_handler:
	di
	push ix:push iy
	push af:push bc:push de:push hl
	exx:ex af,af'
	push af:push bc:push de:push hl
S_curr_interrupt equ $+1:
	ld a,5
	inc a:cp 6
	jr nz,$+3
		xor a
	ld (S_curr_interrupt),a			;save for next time
	ld b,a
	ld a,(display_mode)
	dec b:jr z,interrupt1
	dec b:jr z,interrupt2
	dec b:jr z,interrupt3
	dec b:jr z,interrupt4
	dec b:jr z,interrupt5
	;fall through

interrupt0:
	cp MODE_JUMPNRUN
	call z,draw_animated_tiles
	call screen_switch_buffer
	jr interrupt_end

interrupt1:
	sub 2
	call p,erase_pau_spr	;MODE_TIMESTOP & MODE_JUMPNRUN
	jr interrupt_end

interrupt2:
	cp MODE_JUMPNRUN
	call z,erase_ene_spr
	jr interrupt_end

interrupt3:
	call input_read_active_device
	ld (player_input),a

	ld a,(display_mode)
	or a
	call nz,potion_countup

	rra								;every other frame do alternatively
	ld a,(display_mode)
	jr c,ir3_enemies

	sub 2
	call p,process_pau_mvt			;MODE_TIMESTOP & MODE_JUMPNRUN
	jr interrupt_end

	ir3_enemies:
	cp MODE_JUMPNRUN
	call z,enemies_update				;game mode

	jr interrupt_end


interrupt4:
	or a
	jr z,interrupt_end
	call draw_pau_spr
	call draw_ene_sprs
	jr interrupt_end


interrupt5:
;SOUND (Arkos' AKM Player)
S_change_song equ $+1
	ld a,-1
	inc a:jr z,call_player
		dec a
		ld hl,START
		call PLY_AKM_INIT
		ld a,-1
		ld (S_change_song),a
	call_player:
	call PLY_AKM_PLAY


interrupt_end:
	pop hl:pop de:pop bc:pop af
	exx:ex af,af'
	pop hl:pop de:pop bc:pop af
	pop iy:pop ix
	ei
reti


;#############
;#           #
;# GAME DATA #
;#           #
;#############



_lut_mirror equ #c700
_lut_transparency_masks equ #cf00 ;lookup table for masks, indexed by byte.
 								;AND with screen data, then OR with pixel data.
_lut_scraddr equ #d700
_animated_tiles equ #d7B0
_enemies_curr_stage equ #Df00	;36-40 bytes, no page boundary


read "pau_data.asm"

_comb_tiles:	;storage for the original bg items @this place
	ds 16,#FF

_sprite_orientations:
	ds 8,0		;0=default, -1=mirrored

_inventory:
	ds 13,-1
inv_curr_amount:
	db 0

align #4 ;no page crossing!
_most_frequent:
	ds 4,0

_variables:
pau_spr_orientation:	db 0		;0=right/standard, -1=left
jump_counter:			db 0
wait_frame_after_picking: db 0
pau_drift:				db 0
pau_coords:
pau_y_coord:			db 0
pau_x_coord:			db 0
pau_coords_old:			db 0,0		;y,x
cursor_coords:
cursor_row:				db 0
cursor_col:				db 0

;S_curr_screen_org:		;db 0
frame_counter:			db 0
player_input:			db 0
display_mode:			db 0
	;0 = don't update gfx source addresses when switching screen,
	;	 don't draw anything
	;1 = menu (freeze everything)
	;2 = time stop (only update pau)
	;3 = jump'n'run (update all)


message_byte:			db 0
;for communication between main loop and interruptsS
;BITS:
		;0 = pick up item
		;1 = potion (open page in book) is active
		;2 = update stagemap plz
		;3 = erase countdown plz
		;4 = we did a teleport, load good coords

this_stages_items:		dw 0
amount_items_in_stage:	db 0
this_stages_enemies		dw 0

_gems_in_pocket:			db 0,0,0

diamonds: 				db 0
variables_length equ $-_variables


align #100
_stagemap:
ds TOTAL_TILES

stagemap_end equ $
ds #40,0

end_part1 equ $

org #8000

read "pau_gfx.asm"
_packed_stagemaps:
read "pau_stages.asm"

end_part2 equ $


;save direct "pau1.dat",#40,end_part1-#40
;save direct "pau2.dat",#8000,end_part2-#8000
